import time
from pulsar.apps import tasks
from pulsar.apps.test import unittest

from .test_local import TaskQueueBase



class TestTaskQueueRestart(TaskQueueBase, unittest.TestCase):
    
    def info(self):
        info = yield self.proxy.server_info()
        # get the task queue
        return info['monitors'][self.name()]
        
    def test_kill_task_workers(self):
        tq = yield self.info()
        killed = set()
        for worker in tq['workers']:
            a = worker['actor']
            aid = a['actor_id']
            self.assertEqual(a['is_process'], self.concurrency=='process')
            r = yield self.proxy.kill_actor(aid)
            killed.add(aid)
            self.assertTrue(r)
        # lets get the info again
        start = time.time()
        workers = None 
        while time.time() - start < 5:
            info = yield self.proxy.server_info()
            tq = info['monitors'][self.name()]
            workers = tq['workers']
            if workers:
                break
        self.assertTrue(workers)
        for w in workers:
            a = w['actor']
            self.assertFalse(a['actor_id'] in killed)
            
    def test_check_worker(self):
        r = yield self.proxy.run_new_task(jobname='checkworker')
        self.assertTrue(r)
        r = yield self.proxy.wait_for_task(r)
        self.assertEqual(r['status'], tasks.SUCCESS)
        result = r['result']
        concurrent = result['tasks']
        self.assertTrue(r['id'] in concurrent)
        


'''The example is a :ref:`WSGI application <apps-wsgi>`
with a :ref:`websocket middleware <apps-ws>` which connects to an
IMAP4 server to retrieve and send emails.

The connection with the IMAP4 server is obtained using the IMAP4 API in
twisted 12.3 or later. The example uses
:ref:`pulsar-twisted integration <tutorials-twisted>` module.

To run the server you need to create a :mod:`config.py` file in the
the :mod:`examples.webmail` directory containing::

    # the adress of your mail server
    mail_incoming ='ssl:host=imap.gmail.com:port=993'
    # mail_username & mail_password
    mail_username=
    mail_password=

And type::

    python manage.py
    
Open a web browser at http://localhost:8060 and you should see the web app.

For information on twised IMAP4 client library check this example:

http://twistedmatrix.com/documents/current/mail/examples/imap4client.py

Implementation
==================

.. autoclass:: WsMail
   :members:
   :member-order: bysource
   
'''
import os
import sys
import json
from functools import partial
import time
try:
    import pulsar
except ImportError: #pragma nocover
    sys.path.append('../../')
    import pulsar
from pulsar.utils.pep import zip
from pulsar.apps import ws, wsgi
try:
    from pulsar.apps.tx import twisted
    from twisted.internet import protocol, defer, endpoints, reactor
    from twisted.mail import imap4
except ImportError: #pragma    nocover
    twisted = None    # This is for when we build docs

ASSET_DIR = os.path.join(os.path.dirname(__file__), 'assets')

    
def mail_client(cfg, timeout=10):
    '''Create a new mail client using twisted IMAP4 library.'''
    endpoint = endpoints.clientFromString(reactor, cfg.mail_incoming)
    endpoint._timeout = timeout
    factory = protocol.Factory()
    factory.protocol = imap4.IMAP4Client
    client = yield endpoint.connect(factory)
    yield client.login(cfg.mail_username, cfg.mail_password)
    yield client.select('INBOX')
    yield client
    #info = yield client.fetchEnvelope(imap4.MessageSet(1))
    #print 'First message subject:', info[1]['ENVELOPE'][1]


class WsMail(ws.WS):
    ''':ref:`Websocket handler <websocket-handler>` for fetching and
sending mail via the twisted IMAP4 library.'''
    def on_open(self, request):
        '''When the websocket starts, it create a new mail client.'''
        client = yield mail_client(request.cache.cfg)
        #add the mail client to the environ cache
        request.cache.mailclient = client
        # retrieve the list of mailboxes and them to the client
        yield self._send_mailboxes(request)
        
    def on_message(self, request, msg):
        client = request.cache.get('mailclient')
        if msg and client:
            msg = json.loads(msg)
            if 'mailbox' in msg:
                mailbox = msg['mailbox']
                future = client.examine(mailbox)
                yield future
                result = future.result
                self.write(request, json.dumps({'mailbox': result}))

    def _send_mailboxes(self, request):
        result = yield request.cache.mailclient.list("","*")
        result = sorted([e[2] for e in result])
        self.write(request, json.dumps({'list': result}))


class WebMail(wsgi.LazyWsgi):

    def setup(self):
        return wsgi.WsgiHandler([ws.WebSocket('/message', WsMail()),
                                 wsgi.MediaRouter('/media', ASSET_DIR),
                                 wsgi.Router('/', get=self.home)])

    def home(self, request):
        data = open(os.path.join(ASSET_DIR, 'mail.html')).read()
        request.response.content = data % request.environ
        request.response.content_type = 'text/html'
        return request.response.start()
    
    
def server(name='webmail', callable=None, **kwargs):
    return wsgi.WSGIServer(name=name, callable=WebMail(), **kwargs)


if __name__ == '__main__':  #pragma nocover
    server().start()


'''Routing classes for managing urls.

.. note::

    The :mod:`pulsar.apps.wsgi.route` module was originally from the routing
    module in werkzeug_. Original License:

    copyright (c) 2011 by the Werkzeug Team. License BSD

.. _werkzeug: https://github.com/mitsuhiko/werkzeug

A :class:`Route` is a class for relative url paths::

    r1 = Route('bla')
    r2 = Route('bla/foo')

Integers::

    # accept any integer
    Route('<int:size>')
    # accept an integer between 1 and 200 only
    Route('<int(min=1,max=200):size>')
    

Paths::

    # accept any path (including slashes)
    Route('<path:pages>')
    # accept an integer between 1 and 200 only
    Route('<path:pages>/edit')
    


.. _apps-wsgi-route-decorator:

Route decorator
==================

.. autoclass:: route
   :members:
   :member-order: bysource
   
.. _apps-wsgi-route:

Route
================

.. autoclass:: Route
   :members:
   :member-order: bysource
   
'''
import re

from pulsar import Http404, HttpException
from pulsar.utils.httpurl import iteritems, iri_to_uri, remove_double_slash
from pulsar.utils.httpurl import ENCODE_URL_METHODS, ENCODE_BODY_METHODS
from pulsar.utils.pep import to_string


__all__ = ['route', 'Route']


_rule_re = re.compile(r'''
    (?:
        (?P<converter>[a-zA-Z_][a-zA-Z0-9_]*)   # converter name
        (?:\((?P<args>.*?)\))?                  # converter arguments
        \:                                      # variable delimiter
    )?
    (?P<variable>[a-zA-Z_][a-zA-Z0-9_]*)        # variable name
''', re.VERBOSE)

_converter_args_re = re.compile(r'''
    ((?P<name>\w+)\s*=\s*)?
    (?P<value>
        True|False|
        \d+.\d+|
        \d+.|
        \d+|
        \w+|
        [urUR]?(?P<stringval>"[^"]*?"|'[^']*')
    )\s*,
''', re.VERBOSE|re.UNICODE)


_PYTHON_CONSTANTS = {
    'None':     None,
    'True':     True,
    'False':    False
}


def _pythonize(value):
    if value in _PYTHON_CONSTANTS:
        return _PYTHON_CONSTANTS[value]
    for convert in int, float:
        try:
            return convert(value)
        except ValueError:
            pass
    if value[:1] == value[-1:] and value[0] in '"\'':
        value = value[1:-1]
    return unicode(value)


def parse_rule(rule):
    """Parse a rule and return it as generator. Each iteration yields tuples
    in the form ``(converter, arguments, variable)``. If the converter is
    `None` it's a static url part, otherwise it's a dynamic one.

    :internal:
    """
    m = _rule_re.match(rule)
    if m is None or m.end() < len(rule):
        raise ValueError('Error while parsing rule {0}'.format(rule))
    data = m.groupdict()
    converter = data['converter'] or 'default'
    return converter, data['args'] or None, data['variable']


class route(object):
    '''Decorator for creating a (:class:`Route`, ``HTTPmethod``, ``parameters``)
tuple and assign it to the ``rule_method`` attribute of the ``callable`` which
is decorated. Check the :ref:`HttpBin example <tutorials-httpbin>`
for a sample usage.

:param rule: Optional string for the relative url served by the method which
    is decorated. If not supplied, the method name is used.
:param method: Optional HTTP method name. Default is `get`.
:param defaults: Optional dictionary of default parameters used when
    initialising the :class:`Route` instance.
:param parameters: Additional parameters used when initialising
    the :class:`pulsar.apps.wsgi.handlers.Router` created by this decorator.
    The ``async=True`` parameter will treat the ``callable`` as an asynchronous
    component.
    
The ``callable`` could be an unbound method of a
:ref:`Router <apps-wsgi-router>` class or a standard python function::

    from pulsar.apps import wsgi
    
    class View(wsgi.Router):
    
        wsgi.route('/foo')
        def handle_view(request):
            ...
            
        wsgi.route('/bla', async=True)
        def asynchronous_handle(request):
            result = yield async_function(...)
            yield ...
            
'''
    creation_count = 0
    def __init__(self, rule=None, method=None, defaults=None, **parameters):
        self.__class__.creation_count += 1
        self.creation_count = self.__class__.creation_count
        self.rule = rule
        self.defaults = defaults
        self.method = method
        self.parameters = parameters
        
    def __call__(self, callable):
        bits = callable.__name__.split('_')
        method = None
        if len(bits) > 1:
            m = bits[0].upper()
            if m in ENCODE_URL_METHODS or method in ENCODE_BODY_METHODS:
                method = m
                bits = bits[1:]
        method = (self.method or method or 'get').lower()
        rule = Route(self.rule or '_'.join(bits), defaults=self.defaults)
        callable.rule_method = (rule, method, self.parameters,
                                self.creation_count)
        return callable
        
    
class Route(object):
    '''A Route is a class with a relative :attr:`path`.
    
:parameter rule: Rule strings basically are just normal URL paths
    with placeholders in the format ``<converter(arguments):name>``
    where the converter and the arguments are optional.
    If no converter is defined the `default` converter is used which
    means `string`.
    
:parameter defaults: optional dictionary of default values for variables.
:parameter append_slash: Force a slash to be the last character of the rule.
    In doing so the :attr:`is_leaf` is guaranteed to be ``False``.
    
    
.. attribute:: is_leaf

    If ``True``, the route is equivalent to a file and no sub-routes can be
    added.
    
.. attribute:: path

    The full path for this route including initial ``'/'``.
    
.. attribute:: arguments

    a set of arguments for this route. If the route has no variables, the
    set is empty.
    
.. _werkzeug: https://github.com/mitsuhiko/werkzeug
'''
    def __init__(self, rule, defaults=None):
        rule = remove_double_slash('/%s' % rule)
        self.defaults = defaults if defaults is not None else {}
        self.is_leaf = not rule.endswith('/')
        self.rule = rule[1:]
        self.arguments = set(map(str, self.defaults))
        breadcrumbs = []
        self._converters = {}
        regex_parts = []
        if self.rule:
            for bit in self.rule.split('/'):
                if not bit:
                    continue
                s = bit[0]
                e = bit[-1]
                if s == '<' or e == '>':
                    if s + e != '<>':
                        raise ValueError('malformed rule {0}'.format(self.rule))
                    converter, arguments, variable = parse_rule(bit[1:-1])
                    if variable in self._converters:
                        raise ValueError('variable name {0} used twice\
 in rule {1}.'.format(variable,self.rule))
                    convobj = get_converter(converter, arguments)
                    regex_parts.append('(?P<%s>%s)' % (variable, convobj.regex))
                    breadcrumbs.append((True,variable))
                    self._converters[variable] = convobj
                    self.arguments.add(str(variable))
                else:
                    variable = bit
                    regex_parts.append(re.escape(variable))
                    breadcrumbs.append((False, variable))
                    
        self.breadcrumbs = tuple(breadcrumbs)
        self._regex_string = '/'.join(regex_parts)
        if self._regex_string and not self.is_leaf:
            self._regex_string += '/'
        self._regex = re.compile(self.regex, re.UNICODE)
    
    @property
    def level(self):
        return len(self.breadcrumbs)
        
    @property
    def path(self):
        return '/' + self.rule
    
    @property
    def regex(self):
        if self.is_leaf:
            return '^' + self._regex_string + '$'
        else:
            return '^' + self._regex_string
    
    @property
    def bits(self):
        return tuple((b[1] for b in self.breadcrumbs))
    
    def ordered_variables(self):
        return tuple((b for dyn,b in self.breadcrumbs if dyn))
    
    def __hash__(self):
        return hash(self.rule)
    
    def __repr__(self):
        return self.path
    
    def __eq__(self, other):
        if isinstance(other,self.__class__):
            return str(self) == str(other)
        else:
            return False
        
    def __lt__(self, other):
        if isinstance(other,self.__class__):
            return to_string(self) < to_string(other)
        else:
            raise TypeError('Cannot compare {0} with {1}'.format(self,other))

    def _url_generator(self, values):
        for is_dynamic, val in self.breadcrumbs:
            if is_dynamic:
                val = self._converters[val].to_url(values[val])
            yield val
            
    def url(self, **urlargs):
        '''Build a *url* from *urlargs* dictionary.'''
        if self.defaults:
            d = self.defaults.copy()
            d.update(urlargs)
            urlargs = d
        url = '/'.join(self._url_generator(urlargs))
        if not url:
            return '/'
        else:
            url = '/' + url
            return url if self.is_leaf else url + '/'
        
    def safe_url(self, params=None):
        try:
            if params:
                return self.url(**params)
            else:
                return self.url()
        except KeyError:
            return None
    
    def match(self, path):
        '''Match a path and return ``None`` if no matching, otherwise
 a dictionary of matched variables with values. If there is more
 to be match in the path, the remaining string is placed in the
 ``__remaining__`` key of the dictionary.'''
        match = self._regex.search(path)
        if match is not None:
            remaining = path[match.end():]
            groups = match.groupdict()
            result = {}
            for name, value in iteritems(groups):
                try:
                    value = self._converters[name].to_python(value)
                except Http404:
                    return
                result[str(name)] = value
            if remaining:
                result['__remaining__'] = remaining
            return result
        
    def split(self):
        '''Return a two element tuple containing the parent route and
the last url bit as route. If this route is the root route, it returns
the root route and ``None``. '''
        rule = self.rule
        if not self.is_leaf:
            rule = rule[:-1]
        if not rule:
            return Route('/'),None 
        bits = ('/'+rule).split('/')
        last = Route(bits[-1] if self.is_leaf else bits[-1] + '/')  
        if len(bits) > 1:
            return Route('/'.join(bits[:-1]) + '/'),last
        else:
            return last,None
        
    def __add__(self, other):
        if self.is_leaf:
            raise ValueError('Cannot prepend {0} to {1}. '\
                             'It is a leaf.'.format(self, other))
        cls = self.__class__
        defaults = self.defaults.copy()
        if isinstance(other,cls):
            rule = other.rule
            defaults.update(other.defaults)
        else:
            rule = str(other)
        return cls(self.rule + rule, defaults)
    
    def __deepcopy__(self, memo):
        return self.__copy__()
    
    def __copy__(self):
        return self.__class__(self.rule, self.defaults.copy())
        

class BaseConverter(object):
    """Base class for all converters."""
    regex = '[^/]+'
    weight = 100

    def to_python(self, value):
        return value

    def to_url(self, value):
        return iri_to_uri(value)


class StringConverter(BaseConverter):
    """This converter is the default converter and accepts any string but
    only one path segment.  Thus the string can not include a slash.

    This is the default validator.

    Example::

        Rule('/pages/<page>'),
        Rule('/<string(length=2):lang_code>')

    :param minlength: the minimum length of the string.  Must be greater
                      or equal 1.
    :param maxlength: the maximum length of the string.
    :param length: the exact length of the string.
    """

    def __init__(self, minlength=1, maxlength=None, length=None):
        if length is not None:
            length = '{%d}' % int(length)
        else:
            if maxlength is None:
                maxlength = ''
            else:
                maxlength = int(maxlength)
            length = '{%s,%s}' % (
                int(minlength),
                maxlength
            )
        self.regex = '[^/]' + length


class AnyConverter(BaseConverter):
    """Matches one of the items provided.  Items can either be Python
    identifiers or strings::

        Rule('/<any(about, help, imprint, class, "foo,bar"):page_name>')

    :param items: this function accepts the possible items as positional
                  arguments.
    """

    def __init__(self, *items):
        self.regex = '(?:%s)' % '|'.join([re.escape(x) for x in items])


class PathConverter(BaseConverter):
    """Like the default :class:`StringConverter`, but it also matches
    slashes.  This is useful for wikis and similar applications::

        Rule('/<path:wikipage>')
        Rule('/<path:wikipage>/edit')
    """
    regex = '[^/].*?'
    regex = '.*'
    weight = 200


class NumberConverter(BaseConverter):
    """Baseclass for `IntegerConverter` and `FloatConverter`.

    :internal:
    """
    weight = 1

    def __init__(self, fixed_digits=0, min=None, max=None):
        self.fixed_digits = fixed_digits
        self.min = min
        self.max = max

    def to_python(self, value):
        if (self.fixed_digits and len(value) != self.fixed_digits):
            raise Http404()
        value = self.num_convert(value)
        if (self.min is not None and value < self.min) or \
           (self.max is not None and value > self.max):
            raise Http404()
        return value

    def to_url(self, value):
        if (self.fixed_digits and len(str(value)) > self.fixed_digits):
            raise ValueError()
        value = self.num_convert(value)
        if (self.min is not None and value < self.min) or \
           (self.max is not None and value > self.max):
            raise ValueError()
        if self.fixed_digits: 
            value = ('%%0%sd' % self.fixed_digits) % value
        return str(value)


class IntegerConverter(NumberConverter):
    """This converter only accepts integer values::

        Rule('/page/<int:page>')

    This converter does not support negative values.

    :param fixed_digits: the number of fixed digits in the URL.  If you set
                         this to ``4`` for example, the application will
                         only match if the url looks like ``/0001/``.  The
                         default is variable length.
    :param min: the minimal value.
    :param max: the maximal value.
    """
    regex = r'\d+'
    num_convert = int


class FloatConverter(NumberConverter):
    """This converter only accepts floating point values::

        Rule('/probability/<float:probability>')

    This converter does not support negative values.

    :param min: the minimal value.
    :param max: the maximal value.
    """
    regex = r'\d+\.\d+'
    num_convert = float

    def __init__(self, min=None, max=None):
        super(FloatConverter,self).__init__(0, min, max)


def parse_converter_args(argstr):
    argstr += ','
    args = []
    kwargs = {}

    for item in _converter_args_re.finditer(argstr):
        value = item.group('stringval')
        if value is None:
            value = item.group('value')
        value = _pythonize(value)
        if not item.group('name'):
            args.append(value)
        else:
            name = item.group('name')
            kwargs[name] = value

    return tuple(args), kwargs


def get_converter(name, arguments):
    c = _CONVERTERS.get(name)
    if not c:
        raise LookupError('Route converter {0} not available'.format(name))
    if arguments:
        args, kwargs = parse_converter_args(arguments)
        return c(*args,**kwargs)
    else:
        return c()

    
#: the default converter mapping for the map.
_CONVERTERS = {
    'default':          StringConverter,
    'string':           StringConverter,
    'any':              AnyConverter,
    'path':             PathConverter,
    'int':              IntegerConverter,
    'float':            FloatConverter
}


'''
The :ref:`task queue application <app-taskqueue-app>` does not expose
an external API to run new tasks or retrieve task information.
The :class:`TaskQueueRpcMixin` class can be used to achieve just that. It is
a :class:`pulsar.apps.rpc.JSONRPC` handler which exposes six functions
for executing tasks and retrieving task information.

The :ref:`task-queue example <tutorials-taskqueue>` shows how to use this class
in the context of a WSGI server running along side the task-queue application.

TaskQueue Rpc Mixin
~~~~~~~~~~~~~~~~~~~~~

.. autoclass:: TaskQueueRpcMixin
   :members:
   :member-order: bysource
'''
from collections import Mapping

import pulsar
from pulsar.apps import wsgi
from pulsar.apps import rpc

from .backends import Task, TaskNotAvailable


__all__ = ['TaskQueueRpcMixin']


def task_to_json(task):
    if task:
        if pulsar.is_failure(task):
            err = task.trace[1]
            if isinstance(err, TaskNotAvailable):
                raise rpc.InvalidParams('Job "%s" is not available.'\
                                        % err.task_name)
        if isinstance(task, (list, tuple)):
            task = [task_to_json(t) for t in task]
        elif isinstance(task, Task):
            task = task.tojson()
    return task
    

class TaskQueueRpcMixin(rpc.JSONRPC):
    '''A :class:`pulsar.apps.rpc.JSONRPC` mixin for communicating with
a :class:`TaskQueue`.
To use it, you need to have an :ref:`RPC application <apps-rpc>`
and a :ref:`task queue <apps-taskqueue>` application installed in the
:class:`pulsar.Arbiter`.

:parameter taskqueue: instance or name of the
    :class:`pulsar.apps.tasks.TaskQueue` which exposes the remote procedure
    calls.
    
'''
    _task_backend = None
    def __init__(self, taskqueue, **kwargs):
        if not isinstance(taskqueue, str):
            taskqueue = taskqueue.name
        self.taskqueue = taskqueue
        super(TaskQueueRpcMixin,self).__init__(**kwargs)
        
    ############################################################################
    ##    REMOTES
    def rpc_job_list(self, request, jobnames=None):
        '''Return the list of Jobs registered with task queue with meta
information. If a list of jobnames is given, it returns only jobs
included in the list.'''
        task_backend = yield self.task_backend()
        yield task_backend.job_list(jobnames=jobnames)
    
    def rpc_next_scheduled_tasks(self, request, jobnames=None):
        return self._rq(request, 'next_scheduled', jobnames=jobnames)
        
    def rpc_run_new_task(self, request, jobname=None, **kw):
        '''Run a new task in the task queue. The task can be of any type
as long as it is registered in the task queue registry. To check the
available tasks call the "job_list" function. It returns the task id.'''
        result = yield self.run_new_task(request, jobname, **kw)
        yield task_to_json(result)
        
    def rpc_get_task(self, request, id=None):
        '''Retrieve a task from its id'''
        if id:
            task_backend = yield self.task_backend()
            result = yield task_backend.get_task(id)
            yield task_to_json(result)
    
    def rpc_get_tasks(self, request, **filters):
        '''Retrieve a list of tasks which satisfy key-valued filters'''
        if filters:
            task_backend = yield self.task_backend()
            result = yield task_backend.get_tasks(**filters)
            yield task_to_json(result)
        
    def rpc_wait_for_task(self, request, id=None):
        '''Wait for a task to have finished.'''
        if id:
            task_backend = yield self.task_backend()
            result = yield task_backend.wait_for_task(id)
            yield task_to_json(result)
            
    def rpc_num_tasks(self, request):
        '''Return the approximate number of tasks in the task queue.'''
        task_backend = yield self.task_backend()
        yield task_backend.num_tasks()
    
    ############################################################################
    ##    INTERNALS
    def task_backend(self):
        if not self._task_backend:
            app = yield pulsar.get_application(self.taskqueue)
            self._task_backend = app.backend
        yield self._task_backend
        
    def run_new_task(self, request, jobname, args=None, meta_data=None, **kw):
        if not jobname:
            raise rpc.InvalidParams('"jobname" is not specified!')
        meta_data = meta_data or {}
        meta_data.update(self.task_request_parameters(request))
        args = args or ()
        task_backend = yield self.task_backend()
        yield task_backend.run_job(jobname, args, kw, **meta_data)
    
    def task_request_parameters(self, request):
        '''**Internal function** which returns a dictionary of parameters
to be passed to the :class:`Task` class constructor.
This function can be overridden to add information about
the type of request, who made the request and so forth. It must return
a dictionary. By default it returns an empty dictionary.'''
        return {}
    
    def _rq(self, request, action, *args, **kw):
        return pulsar.send(self.taskqueue, action, *args, **kw)
    



import ctypes
from select import select as _select
try:
    import signal
except ImportError:
    signal = None

from pulsar.utils.importer import import_module, module_attribute
from pulsar.utils.pep import iteritems, native_str

__all__ = ['ALL_SIGNALS',
           'SIG_NAMES',
           'SKIP_SIGNALS',
           'MAXFD',
           'set_proctitle',
           'set_owner_process',
           'IObase',
           'EpollInterface',
           'IOselect']


SIG_NAMES = {}
MAXFD = 1024
SKIP_SIGNALS = frozenset(('KILL', 'STOP', 'WINCH'))

def all_signals():
    if signal:
        for sig in dir(signal):
            if sig.startswith('SIG') and sig[3] != "_":
                val = getattr(signal, sig)
                if isinstance(val, int):
                    name = sig[3:]
                    if name not in SKIP_SIGNALS:
                        SIG_NAMES[val] = name
                        yield name

            
ALL_SIGNALS = tuple(all_signals())


try:
    from setproctitle import setproctitle
    def set_proctitle(title):
        setproctitle(title)
        return True 
except ImportError: #pragma    nocover
    def set_proctitle(title):
        return


def set_owner_process(uid,gid):
    """ set user and group of workers processes """
    if gid:
        try:
            os.setgid(gid)
        except OverflowError:
            # versions of python < 2.6.2 don't manage unsigned int for
            # groups like on osx or fedora
            os.setgid(-ctypes.c_int(-gid).value)
            
    if uid:
        os.setuid(uid)
    

class IObase(object):
    # Constants from the epoll module
    _EPOLLIN = 0x001
    _EPOLLPRI = 0x002
    _EPOLLOUT = 0x004
    _EPOLLERR = 0x008
    _EPOLLHUP = 0x010
    _EPOLLRDHUP = 0x2000
    _EPOLLONESHOT = (1 << 30)
    _EPOLLET = (1 << 31)

    # Our events map exactly to the epoll events
    NONE = 0
    READ = _EPOLLIN
    WRITE = _EPOLLOUT
    ERROR = _EPOLLERR | _EPOLLHUP | _EPOLLRDHUP
        
    
class EpollInterface(object):
    
    def close(self):
        raise NotImplementedError
    
    def fileno(self):
        raise NotImplementedError
    
    def fromfd(self, fd):
        raise NotImplementedError
    
    def register(self, fd, events):
        raise NotImplementedError
    
    def modify(self, fd, events):
        raise NotImplementedError
    
    def unregister(self, fd):
        raise NotImplementedError
    
    def poll(self, timeout=-1):
        raise NotImplementedError
        
        
class IOselect(EpollInterface):
    '''An epoll like select class.'''
    def __init__(self):
        self.read_fds = set()
        self.write_fds = set()
        self.error_fds = set()
        self.fd_dict = (self.read_fds, self.write_fds, self.error_fds)

    def fileno(self):
        return None
    
    def register(self, fd, events):
        if events & IObase.READ:
            self.read_fds.add(fd)
        if events & IObase.WRITE:
            self.write_fds.add(fd)
        if events & IObase.ERROR:
            self.error_fds.add(fd)
            # Closed connections are reported as errors by epoll and kqueue,
            # but as zero-byte reads by select, so when errors are requested
            # we need to listen for both read and error.
            self.read_fds.add(fd)
                
    def modify(self, fd, events):
        self.unregister(fd)
        self.register(fd, events)

    def unregister(self, fd):
        self.read_fds.discard(fd)
        self.write_fds.discard(fd)
        self.error_fds.discard(fd)
            
    def poll(self, timeout=None):
        readable, writeable, errors = _select(
            self.read_fds, self.write_fds, self.error_fds, timeout)
        return self.get_events(readable, writeable, errors)
    
    def get_events(self, readable, writeable, errors):
        events = {}
        for fd in readable:
            events[fd] = events.get(fd, 0) | IObase.READ
        for fd in writeable:
            events[fd] = events.get(fd, 0) | IObase.WRITE
        for fd in errors:
            events[fd] = events.get(fd, 0) | IObase.ERROR
        return list(iteritems(events))
    

Epoll = IOselect



'''\
A Python implementation of JSON-RPC 2.0 Specification

JSON-RPC is a lightweight remote procedure call protocol.
It's designed to be simple.

A remote method is invoked by sending a request to a remote service.
The request is a single object serialized using JSON.

The specification is at http://groups.google.com/group/json-rpc/web/json-rpc-2-0
'''
import sys
import logging
import json
from functools import partial
from timeit import default_timer

import pulsar
from pulsar import async, is_failure, multi_async, log_failure
from pulsar.utils.structures import AttributeDictionary
from pulsar.utils.security import gen_unique_id
from pulsar.utils.pep import to_string, range
from pulsar.utils.jsontools import DefaultJSONEncoder, DefaultJSONHook
from pulsar.utils.tools import checkarity
from pulsar.apps.wsgi import Json
from pulsar.apps.http import HttpClient
from pulsar.utils.httpurl import JSON_CONTENT_TYPES

from .handlers import RpcHandler, InvalidRequest, exception

__all__ = ['JSONRPC', 'JsonProxy', 'LocalJsonProxy']


class JsonToolkit(object):

    @classmethod
    def dumps(cls, data, **kwargs):
        return json.dumps(data, cls=DefaultJSONEncoder, **kwargs)

    @classmethod
    def loads(cls, content):
        return json.loads(content, object_hook=DefaultJSONHook)


class JSONRPC(RpcHandler):
    '''An :class:`RpcHandler` for class for JSON-RPC services.
Design to comply with the `JSON-RPC 2.0`_ Specification.

.. _`JSON-RPC 2.0`: http://www.jsonrpc.org/
'''
    version = '2.0'
    methods = ('post',)
    _json = JsonToolkit

    def __call__(self, request):
        return Json(self._call(request), json=self._json).http_response(request)
    
    @async(max_errors=1)
    def _call(self, request):
        response = request.response
        data = {}
        # No content type - set it equal to application/json
        if not response.content_type:
            response.content_type = 'application/json'
        try:
            try:
                data = request.json_data
            except ValueError:
                raise InvalidRequest(status=415,
                                msg='Content-Type must be application/json')
            if response.content_type not in JSON_CONTENT_TYPES:
                raise InvalidRequest(status=415,
                                msg='Content-Type must be application/json')
            if data.get('jsonrpc') != self.version:
                raise InvalidRequest(
                    'jsonrpc must be supplied and equal to "%s"' % self.version)
            params = data.get('params')
            if isinstance(params, dict):
                args, kwargs = (), params
            else:
                args, kwargs = tuple(params or ()), {}
            #
            callable = self.get_handler(data.get('method'))
            result = yield callable(request, *args, **kwargs)
        except Exception:
            result = yield sys.exc_info()
        #
        res = {'id': data.get('id'), "jsonrpc": self.version}
        if is_failure(result):
            log_failure(result)
            msg = None
            error = result.trace[1]
            if isinstance(error, TypeError):
                msg = checkarity(callable, args, kwargs, discount=1)
            msg = msg or str(error) or 'JSON RPC exception'
            result = {'code': getattr(error, 'fault_code', -32603),
                      'message': msg,
                      'data': getattr(error,'data','')}
            request.response.status_code = getattr(error, 'status', 500)
            res['error'] = result
        else:
            res['result'] = result
        yield res
    

class JsonCall:
    
    dlots = ('_client', '_name')
    
    def __init__(self, client, name):
        self._client = client
        self._name = name
        
    def __repr__(self):
        return self._name
    __str__ = __repr__
    
    @property
    def url(self):
        return self._client.url
    
    @property
    def name(self):
        return self._name
    
    def __getattr__(self, name):
        name = "%s%s%s" % (self._name, self._client.separator, name)
        return self.__class__(self._client, name)
        
    def __call__(self, *args, **kwargs):
        return self._client._call(self._name, *args, **kwargs)
    
        
class JsonProxy(object):
    '''A python Proxy class for :class:`JSONRPC` Servers.

:param url: server location
:param version: JSONRPC server version. Default ``2.0``
:param id: optional request id, generated if not provided. Default ``None``.
:param data: Extra data to include in all requests. Default ``None``.
:param full_response: return the full Http response rather than
    just the content.
:param http: optional http opener. If provided it must have the ``request``
    method available which must be of the form::

        http.request(url, body=..., method=...)

    Default ``None``.

Lets say your RPC server is running at ``http://domain.name.com/``::

    >>> a = JsonProxy('http://domain.name.com/')
    >>> a.add(3,4)
    7
    >>> a.ping()
    'pong'

'''
    separator  = '.'
    rawprefix  = 'raw'
    default_version = '2.0'
    default_timeout = 30
    _json = JsonToolkit

    def __init__(self, url, version=None, data=None, full_response=False, **kw):
        self.__url = url
        self.__version = version or self.__class__.default_version
        self._full_response = full_response
        self.__data = data if data is not None else {}
        self.local = AttributeDictionary()
        self.setup(**kw)

    def setup(self, http=None, timeout=None, **kw):
        if not http:
            timeout = timeout if timeout is not None else self.default_timeout
            http = HttpClient(timeout=timeout, **kw)
        self.local.http = http

    @property
    def url(self):
        return self.__url

    @property
    def http(self):
        return self.local.http
    
    @property
    def version(self):
        return self.__version

    def makeid(self):
        '''Can be re-implemented by your own Proxy'''
        return gen_unique_id()

    def __repr__(self):
        return '%s(%s)' % (self.__class__.__name__, self.__url)

    def __str__(self):
        return self.__repr__()
    
    def __getattr__(self, name):
        return JsonCall(self, name)

    def timeit(self, func, times, *args, **kwargs):
        '''Usefull little utility for timing responses from server. The
usage is simple::

    >>> from pulsar.apps import rpc
    >>> p = rpc.JsonProxy('http://127.0.0.1:8060')
    >>> p.timeit('ping',10)
    0.56...
    >>> _
    '''
        func = getattr(self, func)
        return multi_async((func(*args, **kwargs) for t in range(times)))

    def _call(self, name, *args, **kwargs):
        data, raw = self._get_data(name, *args, **kwargs)
        body = self._json.dumps(data).encode('utf-8')
        # Always make sure the content-type is application/json
        self.http.headers['content-type'] = 'application/json'
        resp = self.http.post(self.url, data=body)
        if self._full_response:
            return resp
        elif hasattr(resp, 'on_finished'):
            res = resp.on_finished.add_callback(partial(self._end_call, raw))
            return res.result if self.http.force_sync else res
        else:
            return self._end_call(raw, resp)
        
    def _end_call(self, raw, resp):
        content = resp.content_string()
        if resp.is_error:
            if 'error' in content:
                return self.loads(content)
            else:
                resp.raise_for_status()
        else:
            if raw:
                return content
            else:
                return self.loads(content)
        
    def _get_data(self, func_name, *args, **kwargs):
        id = self.makeid()
        fs = func_name.split('_')
        raw = False
        if len(fs) > 1 and fs[0] == self.rawprefix:
            raw = True
            fs.pop(0)
            func_name = '_'.join(fs)
        params = self.get_params(*args, **kwargs)
        data = {'method': func_name, 'params': params, 'id': id,
                'jsonrpc': self.__version}
        return data, raw

    def get_params(self, *args, **kwargs):
        '''
        Create an array or positional or named parameters
        Mixing positional and named parameters in one
        call is not possible.
        '''
        kwargs.update(self.__data)
        if args and kwargs:
            raise ValueError('Cannot mix positional and named parameters')
        if args:
            return list(args)
        else:
            return kwargs

    def loads(self, obj):
        res = self._json.loads(obj)
        if isinstance(res, dict):
            if 'error' in res:
                error = res['error']
                raise exception(error.get('code'), error.get('message'))
            else:
                return res.get('result')
        return res


class LocalJsonProxy(JsonProxy):
    '''A proxy class to use when accessing the rpc within the rpc application
domain.'''
    def setup(self, handler=None, environ=None, **kwargs):
        self.local.handler = handler
        self.local.environ = environ

    def _call(self, name, *args, **kwargs):
        data, raw = self._get_data(name, *args, **kwargs)
        hnd = self.local.handler
        environ = self.local.environ
        method, args, kwargs, id, version = hnd.get_method_and_args(data)
        hnd.request(environ, method, args, kwargs, id, version)
        return environ['rpc'].process(Request(environ))



'''Tests the RPC "calculator" example.'''
from pulsar import send
from pulsar.apps import rpc
from pulsar.apps.test import unittest, dont_run_with_thread

from .manage import server


class TestRpcOnThread(unittest.TestCase):
    app = None
    concurrency = 'thread'
    
    @classmethod
    def setUpClass(cls):
        name = 'calc_' + cls.concurrency
        s = server(bind='127.0.0.1:0', name=name, concurrency=cls.concurrency)
        cls.app = yield send('arbiter', 'run', s)
        cls.uri = 'http://{0}:{1}'.format(*cls.app.address)
        cls.p = rpc.JsonProxy(cls.uri)
        cls.sync = rpc.JsonProxy(cls.uri, force_sync=True)
        
    @classmethod
    def tearDownClass(cls):
        if cls.app:
            return send('arbiter', 'kill_actor', cls.app.name)
        
    def setUp(self):
        self.assertEqual(self.p.url, self.uri)
        self.assertTrue(str(self.p))
        proxy = self.p.bla
        self.assertEqual(proxy.name, 'bla')
        self.assertEqual(proxy.url, self.uri)
        self.assertEqual(proxy._client, self.p)
        self.assertEqual(str(proxy), 'bla')
        
    def test_handler(self):
        s = self.app
        self.assertTrue(s.callable)
        wsgi_handler = s.callable.handler
        self.assertEqual(len(wsgi_handler.middleware), 1)
        router = wsgi_handler.middleware[0]
        self.assertEqual(router.route.path, '/')
        root = router.post
        self.assertEqual(len(root.subHandlers), 1)
        hnd = root.subHandlers['calc']
        self.assertFalse(hnd.isroot())
        self.assertEqual(hnd.subHandlers, {})
        
    # Pulsar server commands
    def test_ping(self):
        response = yield self.p.ping()
        self.assertEqual(response, 'pong')
        
    def test_functions_list(self):
        result = yield self.p.functions_list()
        self.assertTrue(result)
        d = dict(result)
        self.assertTrue('ping' in d)
        self.assertTrue('echo' in d)
        self.assertTrue('functions_list' in d)
        self.assertTrue('calc.add' in d)
        self.assertTrue('calc.divide' in d)
        
    def test_time_it(self):
        '''Ping server 20 times'''
        response = self.p.timeit('ping', 20)
        yield response
        self.assertTrue(response.locked_time > 0)
        self.assertTrue(response.total_time > response.locked_time)
        self.assertEqual(response.num_failures, 0)
        
    # Test Object method
    def test_check_request(self):
        result = yield self.p.check_request('check_request')
        self.assertTrue(result)
        
    def testAdd(self):
        response = yield self.p.calc.add(3,7)
        self.assertEqual(response, 10)
        
    def testSubtract(self):
        response = yield self.p.calc.subtract(546, 46)
        self.assertEqual(response, 500)
        
    def testMultiply(self):
        response = yield self.p.calc.multiply(3, 9)
        self.assertEqual(response, 27)
        
    def testDivide(self):
        response = yield self.p.calc.divide(50, 25)
        self.assertEqual(response, 2)
        
    def testInfo(self):
        response = yield self.p.server_info()
        self.assertTrue('server' in response)
        server = response['server']
        self.assertTrue('version' in server)
        
    def testInvalidParams(self):
        self.async.assertRaises(rpc.InvalidParams, self.p.calc.add, 50, 25, 67)
        
    def testInvalidParamsFromApi(self):
        self.async.assertRaises(rpc.InvalidParams, self.p.calc.divide,
                                50, 25, 67)
        
    def test_invalid_function(self):
        p = self.p
        yield self.async.assertRaises(rpc.NoSuchFunction, p.foo, 'ciao')
        yield self.async.assertRaises(rpc.NoSuchFunction,
                                      p.blabla)
        yield self.async.assertRaises(rpc.NoSuchFunction,
                                      p.blabla.foofoo)
        yield self.async.assertRaises(rpc.NoSuchFunction,
                                      p.blabla.foofoo.sjdcbjcb)


        
    def testInternalError(self):
        self.async.assertRaises(rpc.InternalError, self.p.calc.divide,
                                'ciao', 'bo')
        
    def testCouldNotserialize(self):
        self.async.assertRaises(rpc.InternalError, self.p.dodgy_method)
        
    def testpaths(self):
        '''Fetch a sizable ammount of data'''
        response = yield self.p.calc.randompaths(num_paths=20, size=100,
                                                 mu=1, sigma=2)
        self.assertTrue(response)
        
    # Synchronous client
    def test_sync_ping(self):
        self.assertEqual(self.sync.ping(), 'pong')
        self.assertEqual(self.sync.ping(), 'pong')


@dont_run_with_thread
class TestRpcOnProcess(TestRpcOnThread):
    concurrency = 'process'


'''Stand alone compact module for managing python paths.'''
import os
import sys

from .importer import import_module
from .pep import range, string_type


__all__ = ['Path']


class Path(string_type):

    def __new__(cls, path=None):
        path = path or ''
        abspath = os.path.abspath(path)
        return super(Path, cls).__new__(cls, abspath)

    @classmethod
    def cwd(cls):
        '''Return the current working directory as a path object.'''
        return cls(os.getcwd())
        
    isfile = lambda self: os.path.isfile(self)
    isdir = lambda self: os.path.isdir(self)
    exists = lambda self: os.path.exists(self)
    realpath = lambda self: self.__class__(os.path.realpath(string_type(self)))
        
    def join(self, *path):
        return self.__class__(os.path.join(self, *path))
    
    def split(self):
        d,f = os.path.split(self)
        return self.__class__(d), f
    
    def dir(self):
        if self.isfile():
            return self.parent
        elif self.isdir():
            return self
        else:
            raise ValueError('%s not a valid directory' % self)
    
    @property
    def basename(self):
        return os.path.basename(self) 
    
    @property
    def parent(self):
        return self.__class__(os.path.dirname(self))
        
    def module_name(self):
        name = os.path.basename(self)
        if name.endswith('.py'):
            return name[:-3]
        if name.endswith('.pyc'):
            return name[:-4]
        else:
            raise ValueError('%s not a valid python module' % self)
        
    def ancestor(self, n):
        p = self
        for i in range(n):
            p = p.parent
        return p
        
    def ispymodule(self):
        if self.isdir():
            return os.path.isfile(os.path.join(self, '__init__.py'))
        elif self.isfile():
            return self.endswith('.py')
             
    def add2python(self, module=None, up=0, down=None, front=False,
                    must_exist=True):
        '''Add a directory to the python path.
        
:parameter module: Optional module name to try to import once we have found
    the directory
:parameter up: number of level to go up the directory three from
    :attr:`local_path`.
:parameter down: Optional tuple of directory names to travel down once we have
    gone *up* levels.
:parameter front: Boolean indicating if we want to insert the new path at the
    front of ``sys.path`` using ``sys.path.insert(0,path)``.
:parameter must_exist: Boolean indicating if the module must exists.'''
        if module:
            try:
                return import_module(module)
            except ImportError:
                pass
        dir = self.dir().ancestor(up)
        if down:
            dir = dir.join(*down)
        if dir.isdir():
            if dir not in sys.path:
                if front:
                    sys.path.insert(0, dir)
                else:
                    sys.path.append(dir)
        elif must_exist:
            raise ImportError('Directory {0} not available'.format(dir))
        else:
            return None
        if module:
            try:
                return import_module(module)
            except ImportError:
                if must_exist:
                    raise



'''Servers creating Transports'''
import socket
from functools import partial

from pulsar import create_socket, ProtocolError
from pulsar.utils.sockets import create_socket, get_transport_type, wrap_socket
from pulsar.async.defer import EventHandler

from .protocols import Connection, Producer
from .transport import TransportProxy, create_transport, LOGGER

__all__ = ['create_server', 'Server']

     
class Server(Producer, TransportProxy):
    '''A :class:`Producer` for all server's listening for connections.
It is a producer of :class:`Transport` for server protocols.
    
.. attribute:: transport:

    The :class:`Transport` listening for client connections.
    
.. attribute:: consumer_factory

    Callable or a :class:`ProtocolConsumer` class for producing
    :class:`ProtocolConsumer` which handle the receiving, decoding and
    sending of data.
    
.. attribute:: event_loop

    The :class:`EventLoop` running the server.
    
.. attribute:: address

    Server address, where clients send requests to.
    
.. attribute:: timeout

    A timeout in seconds for idle connections
'''
    ONE_TIME_EVENTS = ('start', 'finish')
    MANY_TIMES_EVENTS = ('connection_made', 'pre_request','post_request',
                         'connection_lost')
    consumer_factory = None
    
    def __init__(self, consumer_factory=None, **kw):
        super(Server, self).__init__(**kw)
        if consumer_factory:
            self.consumer_factory = consumer_factory
        
    def data_received(self, sock, address):
        '''Create a new server :class:`Protocol` ready to serve its client.'''
        # Build the connection
        sock = wrap_socket(self.transport.TYPE, sock)
        protocol = self.new_connection(address, self.consumer_factory)
        transport = create_transport(protocol, sock=sock,
                                     event_loop=self.event_loop)
        transport.add_reader()
        protocol.connection_made(transport)
        return protocol
        
    def datagram_received(self, data, address):
        raise NotImplementedError
    
    @property
    def timeout(self):
        return self._timeout
    
    @property
    def address(self):
        return self.transport.address
    
    def connection_made(self, transport):
        self._transport = transport
        transport.add_listener()
        LOGGER.debug('Registered server listening on %s', self)
        self.fire_event('start')
        
    def connection_lost(self, exc):
        self.close_connections()
        self.fire_event('finish')
        
    @classmethod
    def create(cls, event_loop, sock=None, address=None, backlog=1024,
               name=None, close_event_loop=None, **kw):
        '''Create a new server!'''
        sock = create_socket(sock=sock, address=address, bindto=True,
                             backlog=backlog)
        transport_type = get_transport_type(sock.TYPE).transport
        server = cls(**kw)
        transport = transport_type(event_loop, sock, server)
        server.connection_made(transport)
        if close_event_loop:
            server.bind_event('finish',
                                lambda exc: server.event_loop.stop())
        return server

create_server = Server.create



'''Tests the "taskqueue" example.'''
from pulsar.apps.tasks import Task, TaskBackend
from pulsar.apps.test import unittest


class TestTaskClasses(unittest.TestCase):
    
    def testTaskBackend(self):
        b = TaskBackend('dummy', None)
        self.assertRaises(NotImplementedError, b.get_task, 1)
        self.assertRaises(NotImplementedError, b.save_task, 1)
        self.assertRaises(NotImplementedError, b.delete_tasks, [])
        self.assertEqual(b.processed, 0)
        self.assertEqual(b.max_tasks, 0)



'''An :class:`HttpClient` for asynchronous HTTP requests. It can also be used
in synchronous mode by passing ``force_sync`` during initialization::

    >>> from pulsar.apps import http
    >>> client = http.HttpClient()
    >>> response = http.get('http://www.bbc.co.uk')
    
.. _`uri scheme`: http://en.wikipedia.org/wiki/URI_scheme
'''
import os
import platform
import json
from copy import copy
from collections import namedtuple
from base64 import b64encode, b64decode

import pulsar
from pulsar import create_transport, is_failure
from pulsar.utils.pep import native_str, is_string, to_bytes
from pulsar.utils.structures import mapping_iterator
from pulsar.utils.websocket import FrameParser, SUPPORTED_VERSIONS
from pulsar.apps.wsgi import HTTPBasicAuth, HTTPDigestAuth
from pulsar.utils.httpurl import urlparse, urljoin, DEFAULT_CHARSET,\
                                    REDIRECT_CODES, http_parser, httpclient,\
                                    ENCODE_URL_METHODS, parse_qsl,\
                                    encode_multipart_formdata, urlencode,\
                                    Headers, urllibr, get_environ_proxies,\
                                    choose_boundary, urlunparse,\
                                    host_and_port, responses, is_succesful,\
                                    HTTPError, URLError, request_host,\
                                    requote_uri, get_hostport

from .plugins import *


scheme_host = namedtuple('scheme_host', 'scheme netloc')

class TooManyRedirects(Exception):
    pass


class HttpRequest(pulsar.Request):
    '''An :class:`HttpClient` request for an HTTP resource.
    
.. attribute:: method

    The request method
    
.. attribute:: version

    HTTP version for this request, usually ``HTTP/1.1``

.. attribute:: wait_continue

    if ``True``, the :class:`HttpRequest` includes the
    ``Expect: 100-Continue`` header.
    
'''
    full_url = None
    _proxy = None
    def __init__(self, client, url, method, data=None, files=None,
                  charset=None, encode_multipart=True, multipart_boundary=None,
                  timeout=None, hooks=None, history=None, source_address=None,
                  allow_redirects=False, max_redirects=10, decompress=True,
                  version=None, wait_continue=False, websocket_handler=None,
                  **ignored):
        self.client = client
        self.timeout = timeout
        self.method = method.upper()
        self._scheme, self._netloc, self.path, self.params,\
        self.query, self.fragment = urlparse(url)
        if not self._netloc:
            if self.method == 'CONNECT':
                self._netloc = self.path
                self.path = ''
        self.host_only, self.port = get_hostport(self._scheme, self._netloc)
        self.history = history or []
        self.wait_continue = wait_continue
        self.max_redirects = max_redirects
        self.allow_redirects = allow_redirects
        self.charset = charset or 'utf-8'
        self.version = version
        self.decompress = decompress
        self.encode_multipart = encode_multipart 
        self.multipart_boundary = multipart_boundary
        self.websocket_handler = websocket_handler
        self.data = data if data is not None else {}
        self.files = files
        self.source_address = source_address
        self.new_parser()
    
    @property
    def address(self):
        '''``(host, port)`` tuple of the HTTP resource'''
        return (self.host_only, int(self.port))
    
    @property
    def key(self):
        return (self.scheme, self.address, self.timeout)
    
    @property
    def scheme(self):
        '''The `uri scheme`_ of the HTTP resource.'''
        if self._proxy:
            return max(self._scheme, self._proxy.scheme)
        else:
            return self._scheme
        
    @property
    def netloc(self):
        if self._proxy:
            return self._proxy.netloc
        else:
            return self._netloc
        
    def __repr__(self):
        return self.first_line()
    
    def __str__(self):
        return self.__repr__()
    
    def first_line(self):
        url = self._get_url()
        return '%s %s %s' % (self.method, url, self.version)
    
    def new_parser(self):
        self.parser = http_parser(kind=1, decompress=self.decompress)
        
    def set_proxy(self, scheme, host):
        self.host_only, self.port = get_hostport(scheme, host)
        self._proxy = scheme_host(scheme, host)
    
    def encode(self):
        '''Called by :class:`HttpResponse` when it needs to encode this
:class:`HttpRequest` before sending it to the HTTP resourse.
It returns ``bytes``.'''
        buffer = []
        self.body = body = self.encode_body()
        if body:
            self.headers['content-length'] = str(len(body))
            if self.wait_continue:
                self.headers['expect'] = '100-continue'
                body = None
        request = self.first_line()
        buffer.append(request.encode('ascii'))
        buffer.append(b'\r\n')
        buffer.append(bytes(self.headers))
        if body:
            buffer.append(body)
        return b''.join(buffer)
        
    def encode_body(self):
        '''Encode body or url if the :attr:`method` does not have body.
Called by :meth:`encode`.'''
        body = None
        if self.method in ENCODE_URL_METHODS:
            self.files = None
            self._encode_url(self.data)
        elif isinstance(self.data, bytes):
            body = self.data
        elif is_string(self.data):
            body = to_bytes(self.data, self.charset)
        elif self.data:
            content_type = self.headers.get('content-type')
            # No content type given
            if not content_type:
                content_type = 'application/x-www-form-urlencoded'
                if self.encode_multipart:
                    body, content_type = encode_multipart_formdata(
                                            self.data,
                                            boundary=self.multipart_boundary,
                                            charset=self.charset)
                else:
                    body = urlencode(self.data).encode(self.charset)
                self.headers['Content-Type'] = content_type
            elif content_type == 'application/json':
                body = json.dumps(self.data).encode(self.charset)
            else:
                body = json.dumps(self.data).encode(self.charset)
        return body
         
    def all_params(self):
        d = self.__dict__.copy()
        d.pop('client')
        d.pop('method')
        return d
    
    def _encode_url(self, body):
        query = self.query
        if body:
            body = native_str(body)
            if isinstance(body, str):
                body = parse_qsl(body)
            else:
                body = mapping_iterator(body)
            query = parse_qsl(query)
            query.extend(body)
            self.data = query
            query = urlencode(query)
        self.query = query

    def _get_url(self):
        self.full_url = urlunparse((self._scheme, self._netloc, self.path,
                                    self.params, self.query, self.fragment))
        if not self._proxy:
            return urlunparse(('', '', self.path or '/', self.params,
                               self.query, self.fragment))
        else:
            return self.full_url
        
        
class HttpResponse(pulsar.ProtocolConsumer):
    '''A :class:`pulsar.ProtocolConsumer` for the HTTP client protocol.
Initialised by a call to the :class:`HttpClient.request` method.
'''
    _tunnel_host = None
    _has_proxy = False
    _content = None
    
    @property
    def parser(self):
        if self.current_request:
            return self.current_request.parser
    
    def recv_body(self):
        '''Flush the response body and return it.'''
        return self.parser.recv_body()
    
    def get_content(self):
        '''Retrieve the body without flushing'''
        b = self.parser.recv_body()
        if b or self._content is None:
            self._content = self._content + b if self._content else b
        return self._content
    
    def __str__(self):
        return self.status or '<None>'

    def __repr__(self):
        return '%s(%s)' % (self.__class__.__name__, self)

    @property
    def status_code(self):
        if self.parser:
            return self.parser.get_status_code()
    
    @property
    def response(self):
        if self.status_code:
            return responses.get(self.status_code)
        
    @property
    def status(self):
        status_code = self.status_code
        if status_code:
            return '%s %s' % (status_code, responses.get(status_code))
        
    @property
    def url(self):
        if self.current_request is not None:
            return self.current_request.full_url
    
    @property
    def history(self):
        if self.current_request is not None:
           return self.current_request.history
    
    @property
    def headers(self):
        if not hasattr(self, '_headers'):
            if self.parser and self.parser.is_headers_complete():
                self._headers = Headers(self.parser.get_headers())
        return getattr(self, '_headers', None)
    
    @property
    def is_error(self):
        if self.status_code:
            return not is_succesful(self.status_code)
        elif self.on_finished.done():
            return is_failure(self.on_finished.result)
        else:
            return False
    
    def content_string(self, charset=None, errors=None):
        '''Decode content as a string.'''
        data = self.get_content()
        if data is not None:
            return data.decode(charset or 'utf-8', errors or 'strict')

    def content_json(self, charset=None, **kwargs):
        '''Decode content as a JSON object.'''
        return json.loads(self.content_string(charset), **kwargs)
    
    def raise_for_status(self):
        """Raises stored :class:`HTTPError` or :class:`URLError`,
 if one occured."""
        if self.is_error:
            if self.status_code:
                raise HTTPError(self.url, self.status_code,
                                self.content_string(), self.headers, None)
            else:
                raise URLError(self.on_finished.result.trace[1])
    
    def get_origin_req_host(self):
        response = self.history[-1] if self.history else self
        return request_host(request)
    
    ############################################################################
    ##    PROTOCOL IMPLEMENTATION
    def start_request(self):
        self.transport.write(self.current_request.encode())
        
    def data_received(self, data):
        had_headers = self.parser.is_headers_complete()
        if self.parser.execute(data, len(data)) == len(data):
            if self.parser.is_headers_complete():
                if not had_headers:
                    new_response = self.handle_headers()
                    if new_response:
                        had_headers = False
                        return
                if self.parser.is_message_complete():
                    self.finished()
        else:
            raise pulsar.ProtocolError
        
    def close(self):
        if self.parser.is_message_complete():
            self.finished()
            if self.next_url:
                return self.new_request(self.next_url)
            return self
        
    def handle_headers(self):
        '''The response headers are available. Build the response.'''
        request = self.current_request
        headers = self.headers
        client = request.client
        # store cookies in clinet if needed
        if client.store_cookies and 'set-cookie' in headers:
            client.cookies.extract_cookies(self, request)
        # check redirect
        if self.status_code in REDIRECT_CODES and 'location' in headers and\
                request.allow_redirects and self.parser.is_message_complete():
            # done with current response
            url = headers.get('location')
            # Handle redirection without scheme (see: RFC 1808 Section 4)
            if url.startswith('//'):
                parsed_rurl = urlparse(request.full_url)
                url = '%s:%s' % (parsed_rurl.scheme, url)
            # Facilitate non-RFC2616-compliant 'location' headers
            # (e.g. '/path/to/resource' instead of
            # 'http://domain.tld/path/to/resource')
            if not urlparse(url).netloc:
                url = urljoin(request.full_url,
                              # Compliant with RFC3986, we percent
                              # encode the url.
                              requote_uri(url))
            if len(self.history) >= request.max_redirects:
                raise TooManyRedirects
            url = url or self.url
            return client.request(request.method, url, response=self)
        elif self.status_code == 100:
            request.new_parser()
            self.transport.write(request.body)
            return True
        elif self.status_code == 101:
            # Upgrading response handler
            return client.upgrade(self)
    

class HttpClient(pulsar.Client):
    '''A :class:`pulsar.Client` for HTTP/HTTPS servers which handles
a pool of asynchronous :class:`pulsar.Connection`.

.. attribute:: headers

    Default headers for this :class:`HttpClient`.

    Default: :attr:`DEFAULT_HTTP_HEADERS`.

.. attribute:: cookies

    Default cookies for this :class:`HttpClient`

    Default: ``None``.

.. attribute:: timeout

    Default timeout for the connecting sockets. If 0 it is an asynchronous
    client.

.. attribute:: encode_multipart

    Flag indicating if body data is encoded using the ``multipart/form-data``
    encoding by default. It can be overwritten during a :meth:`request`.

    Default: ``True``

.. attribute:: proxy_info

    Dictionary of proxy servers for this client.
    
.. attribute:: DEFAULT_HTTP_HEADERS

    Default headers for this :class:`HttpClient`
    
'''
    consumer_factory = HttpResponse
    allow_redirects = False
    max_redirects = 10
    client_version = 'Python-httpurl'
    version = 'HTTP/1.1' 
    '''Default HTTP request version for this :class:`HttpClient`. It can be
    overwritten during a :meth:`request`.'''
    DEFAULT_HTTP_HEADERS = Headers([
            ('Connection', 'Keep-Alive'),
            ('Accept-Encoding', 'identity'),
            ('Accept-Encoding', 'deflate'),
            ('Accept-Encoding', 'compress'),
            ('Accept-Encoding', 'gzip')],
            kind='client')
    request_parameters = ('encode_multipart', 'max_redirects', 'decompress',
                          'allow_redirects', 'multipart_boundary', 'version',
                          'timeout', 'websocket_handler')
    # Default hosts not affected by proxy settings. This can be overwritten
    # by specifying the "no" key in the proxy_info dictionary
    no_proxy = set(('localhost', urllibr.localhost(), platform.node()))

    def setup(self, proxy_info=None, cache=None, headers=None,
              encode_multipart=True, multipart_boundary=None,
              key_file=None, cert_file=None, cert_reqs='CERT_NONE',
              ca_certs=None, cookies=None, store_cookies=True,
              max_redirects=10, decompress=True, version=None,
              websocket_handler=None):
        self.store_cookies = store_cookies
        self.max_redirects = max_redirects
        self.cookies = cookies
        self.decompress = decompress
        self.version = version or self.version
        dheaders = self.DEFAULT_HTTP_HEADERS.copy()
        dheaders['user-agent'] = self.client_version
        if headers:
            dheaders.update(headers)
        self.headers = dheaders
        self.proxy_info = dict(proxy_info or ())
        if not self.proxy_info and self.trust_env:
            self.proxy_info = get_environ_proxies()
            if 'no' not in self.proxy_info:
                self.proxy_info['no'] = ','.join(self.no_proxy)
        self.encode_multipart = encode_multipart
        self.multipart_boundary = multipart_boundary or choose_boundary()
        self.websocket_handler = websocket_handler
        self.https_defaults = {'key_file': key_file,
                               'cert_file': cert_file,
                               'cert_reqs': cert_reqs,
                               'ca_certs': ca_certs}

    @property
    def websocket_key(self):
        if not hasattr(self, '_websocket_key'):
            self._websocket_key = native_str(b64encode(os.urandom(16)),
                                             'latin-1')
        return self._websocket_key
    
    def get(self, url, **kwargs):
        '''Sends a GET request and returns a :class:`HttpResponse`
object.

:params url: url for the new :class:`HttpRequest` object.
:param \*\*kwargs: Optional arguments for the :meth:`request` method.
'''
        kwargs.setdefault('allow_redirects', True)
        return self.request('GET', url, **kwargs)

    def options(self, url, **kwargs):
        '''Sends a OPTIONS request and returns a :class:`HttpResponse`
object.

:params url: url for the new :class:`HttpRequest` object.
:param \*\*kwargs: Optional arguments for the :meth:`request` method.
'''
        kwargs.setdefault('allow_redirects', True)
        return self.request('OPTIONS', url, **kwargs)

    def head(self, url, **kwargs):
        '''Sends a HEAD request and returns a :class:`HttpResponse`
object.

:params url: url for the new :class:`HttpRequest` object.
:param \*\*kwargs: Optional arguments for the :meth:`request` method.
'''
        return self.request('HEAD', url, **kwargs)

    def post(self, url, **kwargs):
        '''Sends a POST request and returns a :class:`HttpResponse`
object.

:params url: url for the new :class:`HttpRequest` object.
:param \*\*kwargs: Optional arguments for the :meth:`request` method.
'''
        return self.request('POST', url, **kwargs)

    def put(self, url, **kwargs):
        '''Sends a PUT request and returns a :class:`HttpResponse`
object.

:params url: url for the new :class:`HttpRequest` object.
:param \*\*kwargs: Optional arguments for the :meth:`request` method.
'''
        return self.request('PUT', url, **kwargs)

    def patch(self, url, **kwargs):
        '''Sends a PATCH request and returns a :class:`HttpResponse`
object.

:params url: url for the new :class:`HttpRequest` object.
:param \*\*kwargs: Optional arguments for the :meth:`request` method.
'''
        return self.request('PATCH', url, **kwargs)

    def delete(self, url, **kwargs):
        '''Sends a DELETE request and returns a :class:`HttpResponse`
object.

:params url: url for the new :class:`HttpRequest` object.
:param \*\*kwargs: Optional arguments for the :meth:`request` method.
'''
        return self.request('DELETE', url, **kwargs)
    
    def request(self, method, url, cookies=None, headers=None,
                response=None, **params):
        '''Constructs and sends a request to a remote server.
It returns an :class:`HttpResponse` object.

:param method: request method for the :class:`HttpRequest`.
:param url: URL for the :class:`HttpRequest`.
:param params: a dictionary which specify all the optional parameters for
    the :class:`HttpRequest` constructor.

:rtype: a :class:`HttpResponse` object.
'''
        if response:
            rparams = response.current_request.all_params()
            headers = headers or rparams.pop('headers')
            headers.pop('Cookie', None)
            history = copy(rparams['history']) 
            history.append(response.reset_connection())
            # http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html#sec10.3.4
            if response.status_code is 303:
                method = 'GET'
                rparams.pop('data')
                rparams.pop('files')
            params.update(rparams)
            params['history'] = history
        else:
            params = self.update_parameters(self.request_parameters, params)
        request = HttpRequest(self, url, method, **params)
        request.headers = self.get_headers(request, headers)
        self.set_proxy(request)
        if self.cookies:
            self.cookies.add_cookie_header(request)
        if cookies:
            if not isinstance(cookies, CookieJar):
                cookies = cookiejar_from_dict(cookies)
            cookies.add_cookie_header(request)
        #response already available
        if response:
            conn = self.get_connection(request)
            conn.set_consumer(response)
            response.new_request(request)
            return response
        else:
            return self.response(request)
    
    def add_basic_authentication(self, username, password):
        '''Add a :class:`HTTPBasicAuth` handler to the *pre_requests* hooks.'''
        self.bind_event('pre_request', HTTPBasicAuth(username, password))
        
    def add_digest_authentication(self, username, password):
        self.bind_event('pre_request', HTTPDigestAuth(username, password))

    def get_headers(self, request, headers):
        '''Returns a :class:`Header` obtained from combining
:attr:`headers` with *headers*. It handles websocket requests.'''
        if request.scheme in ('ws','wss'):
            d = Headers((('Connection', 'Upgrade'),
                         ('Upgrade', 'websocket'),
                         ('Sec-WebSocket-Version', str(max(SUPPORTED_VERSIONS))),
                         ('Sec-WebSocket-Key', self.websocket_key),
                         ('user-agent', self.client_version)),
                         kind='client')
        else:
            d = self.headers.copy()
        if headers:
            d.update(headers)
        return d
    
    def set_proxy(self, request):
        if request.scheme in self.proxy_info:
            hostonly = request.host_only
            no_proxy = [n for n in self.proxy_info.get('no','').split(',') if n]
            if not any(map(hostonly.endswith, no_proxy)):
                p = urlparse(self.proxy_info[request.scheme])
                request.set_proxy(p.scheme, p.netloc)
                
    def build_protocol(self, address, timeout):
        type, address = address[0], address[1:] 
        return create_connection(address, timeout)
    
    def can_reuse_connection(self, connection, response):
        # Reuse connection only if the headers has Connection keep-alive
        if response and response.headers:
            return response.headers.has('connection', 'keep-alive')
        return False
    
    def upgrade(self, protocol):
        '''Upgrade the protocol to another one'''
        upgrade = protocol.headers['upgrade']
        callable = getattr(self, 'upgrade_%s' % upgrade, None)
        if not callable:
            raise pulsar.ProtocolError
        return callable(protocol.connection, protocol)
    
    def upgrade_websocket(self, connection, handshake):
        '''Upgrade the *protocol* to a websocket response. Invoked
by the :meth:`upgrade` method.'''
        return WebSocketResponse(connection, handshake)
    


'''A web-based chat application, it exposes three different
:ref:`wsgi handlers <wsgi-handlers>`:

* A :class:`pulsar.apps.wsgi.handlers.Router` to render the web page
* A :class:`pulsar.apps.ws.WebSocket` with the :class:`Chat` handler
* A :class:`pulsar.apps.wsgi.handlers.Router` with the :class:`Rpc` handler
  for exposing a :ref:`JSON-RPC <apps-rpc>` api.

To run the server::

    python manage.py
    
and open web browsers at http://localhost:8060

To send messages from the JSON RPC open a python shell and::

    >>> from pulsar.apps import rpc
    >>> p = rpc.JsonProxy('http://127.0.0.1:8060/rpc', force_sync=True)
    >>> p.message('Hi from rpc')
    'OK'
    
Implementation
===========================

.. autoclass:: Chat
   :members:
   :member-order: bysource
   
.. autoclass:: Rpc
   :members:
   :member-order: bysource
   
.. autoclass:: WebChat
   :members:
   :member-order: bysource
'''
import os
import sys
import json
from random import random
import time
try:
    import pulsar
except ImportError: #pragma nocover
    sys.path.append('../../')
    import pulsar
from pulsar.apps import ws, wsgi, rpc, pubsub

CHAT_DIR = os.path.dirname(__file__)

class PubSubClient(pubsub.Client):
    
    def __init__(self, connection):
        self.connection = connection
        
    def __call__(self, channel, message):
        if channel == 'webchat':
            self.connection.write(message)
        

##    Web Socket Chat handler
class Chat(ws.WS):
    '''The websocket handler (:class:`pulsar.apps.ws.WS`) managing the chat
application.

.. attribute:: pubsub

    The :ref:`publish/subscribe handler <apps-pubsub>` created by the wsgi
    application in the :meth:`WebChat.setup` method.
'''
    def __init__(self, pubsub):
        self.pubsub = pubsub
        
    def on_open(self, request):
        '''When a new websocket connection is established it creates a
:ref:`publish/subscribe <apps-pubsub>` client and adds it to the set
of clients of the :attr:`pubsub` handler.'''
        self.pubsub.add_client(PubSubClient(request.cache['websocket']))
        
    def on_message(self, request, msg):
        '''When a new message arrives, it publishes to all listening clients.'''
        if msg:
            lines = []
            for l in msg.split('\n'):
                l = l.strip()
                if l:
                    lines.append(l)
            msg = ' '.join(lines)
            if msg:
                self.pubsub.publish('webchat', msg)


##    RPC MIDDLEWARE To publish messages
class Rpc(rpc.PulsarServerCommands):
    
    def __init__(self, pubsub, **kwargs):
        self.pubsub = pubsub
        super(Rpc, self).__init__(**kwargs)
        
    def rpc_message(self, request, message):
        '''Publish a message via JSON-RPC'''
        self.pubsub.publish('webchat', message)
        return 'OK'
    
    
class WebChat(wsgi.LazyWsgi):
    '''This is the :ref:`wsgi application <wsgi-handlers>` for this
web-chat example.'''
    def setup(self):
        '''This method is called once only to setup the WSGI application
handler as described in :ref:`lazy wsgi handler <wsgi-lazy-handler>`
section. It creates a :ref:`publish/subscribe handler <apps-pubsub>`
and subscribe it to the ``webchat`` channel.'''
        self.pubsub = pubsub.PubSub(encoder=self.encode_message)
        self.pubsub.subscribe('webchat')
        return wsgi.WsgiHandler([wsgi.Router('/', get=self.home_page),
                                 ws.WebSocket('/message', Chat(self.pubsub)),
                                 wsgi.Router('/rpc', post=Rpc(self.pubsub))])
        
    def home_page(self, request):
        data = open(os.path.join(CHAT_DIR, 'chat.html')).read()
        request.response.content_type = 'text/html'
        request.response.content = data % request.environ
        return request.response
    
    def encode_message(self, message):
        if not isinstance(message, dict):
            message = {'message': message}
        message['time'] = time.time()
        return json.dumps(message)


def server(callable=None, **kwargs):
    return wsgi.WSGIServer(callable=WebChat(), **kwargs)


if __name__ == '__main__':  #pragma nocover
    server().start()



import json
import time

from django.shortcuts import render_to_response
from django.template import RequestContext
from django.http import HttpResponse

from pulsar.apps import wsgi, ws, pubsub
from pulsar.utils.structures import AttributeDictionary
from pulsar.utils.log import lazyproperty


def home(request):
    return render_to_response('home.html', {
        'HOST': request.get_host()
        }, RequestContext(request))


class Client(pubsub.Client):
    
    def __init__(self, connection):
        self.connection = connection
        
    def __call__(self, channel, message):
        if channel == 'webchat':
            self.connection.write(message)


class Chat(ws.WS):
    '''The websocket handler (:class:`pulsar.apps.ws.WS`) managing the chat
application.'''
    @lazyproperty
    def pubsub(self):
        p = pubsub.PubSub()
        p.subscribe('webchat')
        return p
            
    def on_open(self, request):
        '''When a new websocket connection is established it add connection
to the set of clients listening for messages.'''
        self.pubsub.add_client(Client(request.cache['websocket']))
        
    def on_message(self, request, msg):
        '''When a new message arrives, it publishes to all listening clients.'''
        if msg:
            lines = []
            for l in msg.split('\n'):
                l = l.strip()
                if l:
                    lines.append(l)
            msg = ' '.join(lines)
            if msg:
                user = request.get('django.cache').user
                if user.is_authenticated():
                    user = user.username
                else:
                    user = 'anonymous'
                msg = {'message': msg, 'user': user, 'time': time.time()}
                self.pubsub.publish('webchat', json.dumps(msg))
                

class middleware(object):
    '''Middleware for serving the Chat websocket'''
    def __init__(self):
        self._web_socket = ws.WebSocket('/message', Chat(),
                                        start_response=False)
        
    def process_request(self, request):
        data = AttributeDictionary(request.__dict__)
        environ = data.pop('environ')
        environ['django.cache'] = data
        response = self._web_socket(environ, None)
        if response is not None:
            # Convert to django response
            resp = HttpResponse(status=response.status_code,
                                content_type=response.content_type)
            for header, value in response.headers:
                resp[header] = value
            return resp


import datetime
import os
import subprocess

def get_version(version):
    assert len(version) == 5
    assert version[3] in ('alpha', 'beta', 'rc', 'final')
    main = '.'.join(map(str, version[:3]))
    sub = ''
    if version[3] == 'alpha' and version[4] == 0:
        git_changeset = get_git_changeset()
        if git_changeset:
            sub = '-dev.%s' % git_changeset
    elif version[3] != 'final':
        sub = '-%s.%s' % tuple(version[3:])
    return main + sub

def get_git_changeset():
    """Returns a numeric identifier of the latest git changeset.

    The result is the UTC timestamp of the changeset in YYYYMMDDHHMMSS format.
    This value isn't guaranteed to be unique, but collisions are very unlikely,
    so it's sufficient for generating the development version numbers.
    """
    repo_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    git_show = subprocess.Popen('git show --pretty=format:%ct --quiet HEAD',
            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
            shell=True, cwd=repo_dir, universal_newlines=True)
    timestamp = git_show.communicate()[0].partition('\n')[0]
    try:
        timestamp = datetime.datetime.utcfromtimestamp(int(timestamp))
    except ValueError:
        return None
    return timestamp.strftime('%Y%m%d%H%M%S')


import time

import pulsar
from pulsar.apps.test import unittest, run_on_arbiter
from pulsar.apps.shell import InteractiveConsole, decode_line, PulsarShell


class DummyConsole(InteractiveConsole):
    
    def setup(self):
        pass
    
    def interact(self, timeout):
        time.sleep(timeout)
        
def start(actor):
    return PulsarShell(console_class=DummyConsole, workers=2).start()
    

class TestShell(unittest.TestCase):
    app = None
    @classmethod
    def setUpClass(cls):
        cls.app = yield pulsar.send('arbiter', 'run', start)
        
    @classmethod
    def tearDownClass(cls):
        if cls.app is not None:
            yield pulsar.send('arbiter', 'kill_actor', cls.app.name)
            
    def testApp(self):
        app = self.app
        self.assertEqual(app.name, 'shell')
        self.assertEqual(app.callable, None)
        self.assertEqual(app.cfg.console_class, DummyConsole)
        #self.assertEqual(app.cfg.workers, 1)
        #self.assertEqual(app.cfg.concurrency, 'thread')
        self.assertEqual(decode_line('bla'), 'bla')
        
    @run_on_arbiter
    def testTestWorker(self):
        arbiter = pulsar.get_actor()
        monitor = arbiter.get_actor('shell')
        while not monitor.managed_actors:
            yield pulsar.NOT_DONE
        self.assertEqual(len(monitor.managed_actors), 1)


'''\
This is a a :ref:`JSON-RPC <apps-rpc>` server with some simple functions.
To run the server type::

    python manage.py

Open a new shell and launch python and type::

    >>> from pulsar.apps import rpc
    >>> p = rpc.JsonProxy('http://localhost:8060', force_sync=True)
    >>> p.ping()
    'pong'
    >>> p.functions_list()
    [[...
    >>> p.calc.add(3,4)
    7.0

The ``force_sync`` keyword is used here to force the Json RPC client to
wait for a full response rather than returning a :class:`pulsar.Deferred`.
Check the :ref:`creating synchronous clients <tutorials-synchronous>` tutorial. 

Implementation
-----------------

The calculator rpc functions are implemented by the :class:`Calculator`
handler, while the :class:`Root` handler exposes utility methods from
the :class:`pulsar.apps.rpc.PulsarServerCommands` handler.

.. autoclass:: Calculator
   :members:
   :member-order: bysource
   
.. autoclass:: Site
   :members:
   :member-order: bysource

'''
try:
    import pulsar
except ImportError: #pragma nocover
    import sys
    sys.path.append('../../')
    
from random import normalvariate

from pulsar.apps import rpc, wsgi
from pulsar.utils.httpurl import range, JSON_CONTENT_TYPES


def divide(request, a, b):
    '''Divide two numbers. This method illustrate how to use the
:func:`pulsar.apps.rpc.rpc_method` decorator.'''
    return float(a)/float(b)

def request_handler(request, format, kwargs):
    '''Dummy request handler'''
    return kwargs

def randompaths(request, num_paths=1, size=250, mu=0, sigma=1):
    '''Lists of random walks.'''
    r = []
    for p in range(num_paths):
        v = 0
        path = [v]
        r.append(path)
        for t in range(size):
            v += normalvariate(mu, sigma)
            path.append(v)
    return r


class RequestCheck:
    
    def __call__(self, request, name):
        assert(request.json_data['method'] == name)
        return True


class Root(rpc.PulsarServerCommands):
    
    def rpc_dodgy_method(self, request):
        '''This method will fails because the return object is not
json serializable.'''
        return Calculator
    
    rpc_check_request = RequestCheck()


class Calculator(rpc.JSONRPC):
    '''A :class:`pulsar.apps.rpc.JSONRPC` handler which implements few simple
remote functions.'''
    def rpc_add(self, request, a, b):
        '''Add two numbers'''
        return float(a) + float(b)

    def rpc_subtract(self, request, a, b):
        '''Subtract two numbers'''
        return float(a) - float(b)

    def rpc_multiply(self, request, a, b):
        '''Multiply two numbers'''
        return float(a) * float(b)

    rpc_divide = rpc.rpc_method(divide, request_handler=request_handler)
    rpc_randompaths = rpc.rpc_method(randompaths)


class Site(wsgi.LazyWsgi):
    '''WSGI handler for the RPC server'''
    def setup(self):
        '''Called once to setup the list of wsgi middleware.'''
        json_handler = Root().putSubHandler('calc', Calculator())
        middleware = wsgi.Router('/', post=json_handler,
                                 accept_content_types=JSON_CONTENT_TYPES)
        response = [wsgi.GZipMiddleware(200)]
        return wsgi.WsgiHandler(middleware=[middleware],
                                response_middleware=response)
    

def server(callable=None, **params):
    return wsgi.WSGIServer(Site(), **params)


if __name__ == '__main__':  #pragma nocover
    server().start()




'''
A test plugin for benchmarking test cases.
'''
import sys
import time
import math

if sys.platform == "win32": #pragma    nocover
    default_timer = time.clock
else:
    default_timer = time.time
    
import pulsar
from pulsar.utils.httpurl import range

from pulsar.apps import test

BENCHMARK_TEMPLATE = '\nRepeated {0[number]} times.\
 Average {0[mean]} secs, Stdev {0[std]}.'

class BenchTest(test.WrapTest):
    __benchmark__ = True
    def __init__(self, test, number):
        super(BenchTest, self).__init__(test)
        self.number = number
        
    def updateSummary(self, info, number, total_time, total_time2):
        mean = total_time/number
        std = math.sqrt((total_time2 - total_time*mean)/number)
        std = round(100*std/mean,2)
        info.update({'number': number,
                     'mean': '%.5f' % mean,
                     'std': '{0} %'.format(std)})
        
    def _call(self):
        simple = lambda info, *args: info
        testMethod = self.testMethod
        testStartUp = getattr(self.test, 'startUp', lambda : None)
        testGetTime = getattr(self.test, 'getTime', lambda dt : dt)
        testGetInfo = getattr(self.test, 'getInfo', simple)
        testGetSummary = getattr(self.test, 'getSummary', simple)
        t = 0
        t2 = 0
        info = {}
        for r in range(self.number):
            testStartUp()
            start = default_timer()
            yield testMethod()
            delta = default_timer() - start
            dt = testGetTime(delta)
            testGetInfo(info, delta, dt)
            t += dt
            t2 += dt*dt
        self.updateSummary(info, self.number, t, t2)
        self.set_test_attribute('bench_info',
                                testGetSummary(info, self.number, t, t2))


class BenchMark(test.TestPlugin):
    '''Benchmarking addon for pulsar test suite.'''
    desc = '''Run benchmarks function flagged with __benchmark__ attribute'''
    
    repeat = pulsar.Setting(flags=['--repeat'],
                            type=int,
                            default=1,
                            validator=pulsar.validate_pos_int,
                            desc='Default number of repetition '\
                                 'when benchmarking.''')
        
    def before_test_function_run(self, test, local):
        if self.config.benchmark: 
            method_name = getattr(test, '_testMethodName', None)
            if method_name:
                method = getattr(test, method_name, None)
                bench = getattr(test, '__benchmark__', False)
                if not bench and method:
                    bench = getattr(method, '__benchmark__', False)
                if bench:
                    number = getattr(test, '__number__', self.config.repeat)
                    return BenchTest(test, number)
    
    def addSuccess(self, test):
        if self.config.benchmark and self.stream:
            result = getattr(test, 'bench_info', None)
            if result and self.stream.showAll:
                stream = self.stream.handler('benchmark')
                template = getattr(test, 'benchmark_template',
                                   BENCHMARK_TEMPLATE)
                stream.writeln(template.format(result))
                stream.flush()
                return True



"""Multi-consumer multi-producer dispatching mechanism,
originally based on pydispatch_ (BSD).

To register a new *event* use the :func:`bind` function::

    from pulsar.utils import events
    
    def handler(signal, sender, data=None, **params):
        ...
    
    events.bind('connection_lost', handler)
    
.. _pydispatch:  http://pypi.python.org/pypi/PyDispatcher/2.0.1
"""
import logging
from .dispatcher import Signal


LOGGER = logging.getLogger('pulsar.events')

_events = {}

def create(name, providing_args=None):
    '''Create a new event from a *name*. If the event is already available
it returns it.'''
    name = name.lower()
    if name not in _events:
        event = Signal()
        event.name = name
        _events[name] = event
    return _events[name]

def bind(name, callback, sender=None, **kwargs):
    '''Bind a *callback* to event *name*. The optional *sender* can be used
to bind only when the given sender fires the event.'''
    event = create(name)
    event.connect(callback, sender=sender, **kwargs)
    
def unbind(name, callback, sender=None, **kwargs):
    '''Bind a *callback* to event *name*. The optional *sender* can be used
to bind only when the given sender fires the event.'''
    event = create(name)
    event.disconnect(callback, sender=sender, **kwargs)
    
def fire(name, sender, **kwargs):
    '''Fire event *name* from *sender*.'''
    event = create(name)
    event.send(sender, **kwargs)
    
    
class Listener(object):
    '''A class for testing signals'''
    def __init__(self, *events):
        self._events = {}
        for event in events:
            self._events[event] = []
            bind(event, self)
            
    def __call__(self, signal=None, sender=None, data=None, **kwargs):
        self._events[signal.name].append((sender, data))
        
    def __getitem__(self, event):
        return self._events[event]


'''Actors communicate with each other by sending and receiving messages.
The :mod:`pulsar.async.mailbox` module implements the message passing layer
via a bidirectional socket connections between the :class:`pulsar.Arbiter`
and :class:`pulsar.Actor`.

Message sending is asynchronous and safe, the message is guaranteed to
eventually reach the recipient, provided that the recipient exists.

The implementation details are outlined below:

* Messages are sent via the :func:`send` function, which is a proxy for
  the :class:`pulsar.Actor.send` method. Here is how you ping actor ``abc``::
      
      from pulsar import send
      
      send('abc', 'ping')
        
* The :class:`pulsar.Arbiter` mailbox is a TCP :class:`pulsar.Server`
  accepting :class:`pulsar.Connection` from remote actors.
* The :attr:`pulsar.Actor.mailbox` is a :class:`pulsar.Client` of the arbiter
  mailbox server.
* When an actor sends a message to another actor, the arbiter mailbox behaves
  like a proxy server by routing the message to the targeted actor.
* Communication is bidirectional and there is **only one connection** between
  the arbiter and any given actor.
* Messages are encoded and decoded using the unmasked websocket protocol
  implemented in :class:`pulsar.utils.websocket.FrameParser`.
* If, for some reasons, the commonication between an actor and the arbiter
  get broken, the actor will eventually stop running and garbaged collected.
* The :attr:`Actor.mailbox` for :ref:`CPU bound actors <cpubound>` has its hown
  separate :class:`EventLoop` on a separate thread of execution. In other
  words, it does not share it with the actor so that I/O requests
  can be processed even if the actor is performing a long calculation.
  
  
Actor mailbox protocol
=========================
  For the curious this is how the internal protocol is implemented
  
  .. autoclass:: MailboxConsumer
   :members:
   :member-order: bysource
'''
import sys
import logging
import tempfile
from functools import partial
from collections import namedtuple

from pulsar import platform, Config, ProtocolError, CommandError
from pulsar.utils.pep import to_bytes, ispy3k, ispy3k, pickle, set_event_loop,\
                             new_event_loop
from pulsar.utils.sockets import nice_address
from pulsar.utils.websocket import FrameParser
from pulsar.utils.security import gen_unique_id

from .access import get_actor, set_actor
from .defer import async, maybe_failure, log_failure, Deferred
from .transports import ProtocolConsumer, SingleClient, Request
from .proxy import actorid, get_proxy, get_command, ActorProxy


LOGGER = logging.getLogger('pulsar.mailbox')
    
CommandRequest = namedtuple('CommandRequest', 'actor caller connection')

def command_in_context(command, caller, actor, args, kwargs):
    cmnd = get_command(command)
    if not cmnd:
        raise CommandError('unknown %s' % command)
    request = CommandRequest(actor, caller, None)
    return cmnd(request, args, kwargs)
    
    
class MonitorMailbox(object):
    '''A :class:`Mailbox` for a :class:`Monitor`. This is a proxy for the
arbiter mailbox.'''
    active_connections = 0
    def __init__(self, actor):
        self.mailbox = actor.monitor.mailbox
        # make sure the monitor get the hand shake!
        self.mailbox.event_loop.call_soon_threadsafe(actor.hand_shake)

    def __repr__(self):
        return self.mailbox.__repr__()
    
    def __str__(self):
        return self.mailbox.__str__()
    
    def __getattr__(self, name):
        return getattr(self.mailbox, name)
    
    def _run(self):
        pass
    
    def close(self):
        pass
    

class Message(Request):
    '''A message which travels from actor to actor.'''
    def __init__(self, data, future=None, address=None, timeout=None):
        super(Message, self).__init__(address, timeout)
        self.data = data
        self.future = future
    
    @classmethod
    def command(cls, command, sender, target, args, kwargs, address=None,
                 timeout=None):
        command = get_command(command)
        data = {'command': command.__name__,
                'sender': actorid(sender),
                'target': actorid(target),
                'args': args if args is not None else (),
                'kwargs': kwargs if kwargs is not None else {}}
        if command.ack:
            future = Deferred()
            data['ack'] = gen_unique_id()[:8]
        else:
            future = None
        return cls(data, future, address, timeout)
        
    @classmethod
    def callback(cls, result, ack):
        data = {'command': 'callback', 'result': result, 'ack': ack}
        return cls(data)
        

class MailboxConsumer(ProtocolConsumer):
    '''The :class:`pulsar.ProtocolConsumer` for internal message passing
between actors. Encoding and decoding uses the unmasked websocket
protocol.'''
    def __init__(self, *args, **kwargs):
        super(MailboxConsumer, self).__init__(*args, **kwargs)
        self._pending_responses = {}
        self._parser = FrameParser(kind=2)
    
    def request(self, command, sender, target, args, kwargs):
        '''Used by the server to send messages to the client.'''
        req = Message.command(command, sender, target, args, kwargs)
        self.new_request(req)
        return req.future
    
    ############################################################################
    ##    PROTOCOL CONSUMER IMPLEMENTATION
    def data_received(self, data):
        # Feed data into the parser
        msg = self._parser.decode(data)
        while msg:
            try:
                message = pickle.loads(msg.body)
            except Exception:
                raise ProtocolError('Could not decode message body')
            log_failure(self._responde(message))
            msg = self._parser.decode()
    
    def start_request(self):
        # Always called in thread
        req = self.current_request
        if req.future and 'ack' in req.data:
            self._pending_responses[req.data['ack']] = req.future
            try:
                self._write(req)
            except Exception as e:
                req.future.callback(e)
        else:
            self._write(req)
    
    ############################################################################
    ##    INTERNALS
    @async(max_errors=None)
    def _responde(self, message):
        actor = get_actor()
        command = message.get('command')
        #LOGGER.debug('%s handling message "%s"', actor, command)
        if command == 'callback':   #this is a callback
            self._callback(message.get('ack'), message.get('result'))
        else:
            try:
                target = actor.get_actor(message['target'])
                if target is None:
                    raise CommandError('Cannot execute "%s" in %s. Unknown '
                            'actor %s' % (command, actor, message['target']))
                caller = get_proxy(actor.get_actor(message['sender']), safe=True)
                if isinstance(target, ActorProxy):
                    # route the message to the actor proxy
                    if caller is None:
                        raise CommandError("'%s' got message from unknown '%s'" %
                                           (actor, message['sender']))
                    result = yield actor.send(target, command, *message['args'],
                                              **message['kwargs'])
                else:
                    actor = target
                    command = get_command(command)
                    req = CommandRequest(target, caller, self.connection)
                    result = yield command(req, message['args'], message['kwargs'])
            except Exception as e:
                result = maybe_failure(e)
            yield self._response(message, result)
        
    def _response(self, data, result):
        if data.get('ack'):
            req = Message.callback(result, data['ack'])
            self.new_request(req)
        #Return the result so a failure can be logged
        return result

    def _callback(self, ack, result):
        if not ack:
            raise ProtocolError('A callback without id')
        try:
            pending = self._pending_responses.pop(ack)
        except KeyError:
            raise KeyError('Callback %s not in pending callbacks' % ack)
        pending.callback(result)
        
    def _write(self, req):
        obj = pickle.dumps(req.data, protocol=2)
        data = self._parser.encode(obj, opcode=0x2).msg
        self.transport.write(data)
        
    
class MailboxClient(SingleClient):
    # mailbox for actors client
    consumer_factory = MailboxConsumer
    max_reconnect = 0
     
    def __init__(self, address, actor, event_loop):
        super(MailboxClient, self).__init__(address, event_loop=event_loop)
        self.name = 'Mailbox for %s' % actor
        # when the mailbox shutdown, the event loop must stop.
        self.bind_event('finish', lambda s: s.event_loop.stop())
    
    def __repr__(self):
        return '%s %s' % (self.__class__.__name__, nice_address(self.address))
    
    def request(self, command, sender, target, args, kwargs):
        # the request method
        req = Message.command(command, sender, target, args, kwargs,
                              self.address, self.timeout)
        # we make sure responses are run on the event loop thread
        self.event_loop.call_now_threadsafe(self.response, req)
        return req.future
        


import os
import sys
import logging
import traceback
import inspect
import errno
import socket
from heapq import heappush, heappop
from functools import partial
from collections import deque
from threading import current_thread
try:
    import signal
except ImportError: #pragma    nocover
    signal = None

from pulsar.utils.system import IObase, IOpoll, close_on_exec, platform, Waker
from pulsar.utils.pep import default_timer, set_event_loop_policy,\
                             set_event_loop, new_event_loop, get_event_loop,\
                             EventLoop as BaseEventLoop, range,\
                             EventLoopPolicy as BaseEventLoopPolicy
from pulsar.utils.sockets import SOCKET_INTERRUPT_ERRORS

from .access import thread_local_data
from .defer import log_failure, is_failure, Deferred, TimeoutError, maybe_async
from .transports import create_server

__all__ = ['EventLoop', 'TimedCall']

LOGGER = logging.getLogger('pulsar.eventloop')


def file_descriptor(fd):
    if hasattr(fd, 'fileno'):
        return fd.fileno()
    else:
        return fd

def setid(self):
    ct = current_thread()
    self.tid = ct.ident
    self.pid = os.getpid()
    return ct


class StopEventLoop(BaseException):
    """Raised to stop the event loop."""
    
    
class EventLoopPolicy(BaseEventLoopPolicy):
    '''Pulsar event loop policy'''
    def get_event_loop(self):
        return thread_local_data('_event_loop')
    
    def get_request_loop(self):
        return thread_local_data('_request_loop') or self.get_event_loop()
    
    def new_event_loop(self, **kwargs):
        return EventLoop(**kwargs)
    
    def set_event_loop(self, event_loop):
        """Set the event loop."""
        assert event_loop is None or isinstance(event_loop, BaseEventLoop)
        if getattr(event_loop, 'cpubound', False):
            thread_local_data('_request_loop', event_loop)
        else:
            thread_local_data('_event_loop', event_loop)
        
    
set_event_loop_policy(EventLoopPolicy())
                
            
class TimedCall(object):
    """An EventLoop callback handler. This is not initialised directly, instead
it is created by :meth:`EventLoop.call_soon`, :meth:`EventLoop.call_later`,
:meth:`EventLoop.call_soon_threadsafe` and so forth.
    
.. attribute:: deadline

    a time in the future or ``None``.
    
.. attribute:: callback

    The callback to execute in the eventloop
    
.. attribute:: cancelled

    Flag indicating this callback is cancelled.
    """

    def __init__(self, deadline, callback, args):
        self.reschedule(deadline)
        self._callback = callback
        self._args = args

    def __lt__(self, other):
        return self.deadline < other.deadline
        
    @property
    def deadline(self):
        return self._deadline
    
    @property
    def cancelled(self):
        return self._cancelled
    
    @property
    def callback(self):
        return self._callback
    
    @property
    def args(self):
        return self._args
    
    def cancel(self):
        '''Attempt to cancel the callback.'''
        self._cancelled = True
                
    def reschedule(self, new_deadline):
        self._deadline = new_deadline
        self._cancelled = False
    
    def __call__(self, *args, **kwargs):
        if not self._cancelled:
            args = self._args + args
            return self._callback(*args, **kwargs)
        
        
class FileDescriptor(IObase):
    def __init__(self, fd, eventloop):
        self.fd = fd
        self.eventloop = eventloop
        self.logger = eventloop.logger
        self.handle_write = None
        self.handle_read = None

    @property
    def poller(self):
        return self.eventloop._impl
    
    @property
    def reading(self):
        return bool(self.handle_read)
    
    @property
    def writing(self):
        return bool(self.handle_write)
    
    @property
    def state(self):
        if self.reading:
            if self.writing:
                return self.READ | self.WRITE
            else:
                return self.READ
        elif self.writing:
            return self.WRITE
            
    @property
    def state_code(self):
        s = []
        if self.state is None:
            return 'closed'
        if self.connecting:
            s.append('connecting')
        elif self.writing:
            s.append('writing')
        if self.reading:
            s.append('reading')
        return ' '.join(s) if s else 'idle'
    
    def add_connector(self, callback):
        self.add_writer(callback, 'connector')
        
    def add_reader(self, callback):
        handle_read = self.handle_read
        if handle_read:
            if handle_read.callback != callback.callback:
                raise RuntimeError('Read handler already registered')
        else:
            self.logger.debug('Add reader on file descriptor %s', self.fd)
            current_state = self.state
            self.handle_read = callback
            self.modify_state(current_state, self.READ)
        
    def add_writer(self, callback, name=None):
        name = name or 'writer'
        handle_write = self.handle_write
        if handle_write:
            if handle_write.callback != callback.callback:
                raise RuntimeError('%s handler already registered', name)
        else:
            self.logger.debug('Add %s on file descriptor %s', name, self.fd)
            current_state = self.state
            self.handle_write = callback
            self.modify_state(current_state, self.WRITE)
        
    def remove_connector(self):
        return self.remove_writer('connector')
    
    def remove_reader(self):
        '''Remove reader and return True if writing'''
        if self.writing:
            self.logger.debug('Remove reader from file descriptor %s', self.fd)
            self.poller.modify(self.fd, self.WRITE)
        else:
            self.remove_handler()
        self.handle_read = None

    def remove_writer(self, name=None):
        '''Remove writer and return True if reading'''
        if self.reading:
            self.logger.debug('Remove %s from file descriptor %s',
                              name or 'writer', self.fd)
            self.poller.modify(self.fd, self.READ)
        else:
            self.remove_handler()
        self.handle_write = None
    
    def __call__(self, events):
        if events & self.READ:
            if self.handle_read:
                log_failure(self.handle_read())
            else:
                self.logger.warning('Read callback without handler for file'
                                    ' descriptor %s.', self.fd)
        if events & self.WRITE:
            if self.handle_write:
                log_failure(self.handle_write())
            else:
                self.logger.warning('Write callback without handler for file'
                                    ' descriptor %s.', self.fd)
            
    def modify_state(self, current_state, state):
        if current_state != state:
            if current_state is None:
                self.poller.register(self.fd, state)
            else:
                self.poller.modify(self.fd, current_state | state)
    
    def remove_handler(self):
        """Stop listening for events on fd."""
        fd = self.fd
        try:
            self.poller.unregister(fd)
        except (OSError, IOError):
            self.eventloop.logger.error("Error removing %s from EventLoop", fd)
        self.logger.debug('Remove file descriptor %s', fd)
        self.eventloop._handlers.pop(fd, None)


class LoopingCall(object):
    
    def __init__(self, event_loop, callback, args, interval=None):
        self.event_loop = event_loop
        self.callback = callback
        self.args = args
        self._cancelled = False
        interval = interval or 0
        if interval > 0:
            self.interval = interval
            self.handler = self.event_loop.call_later(interval, self)
        else:
            self.interval = None
            self.handler = self.event_loop.call_soon(self)
    
    @property
    def cancelled(self):
        return self._cancelled
    
    def cancel(self, result=None):
        '''Attempt to cancel the callback.'''
        self._cancelled = True
        log_failure(result)
        
    def __call__(self):
        # If this fails, the chain is broken.
        try:
            result = maybe_async(self.callback(*self.args), get_result=False)
        except Exception:
            self.cancel(sys.exc_info())
        else:
            result.add_callback(self._continue, self.cancel)
        
    def _continue(self, result):
        if not self._cancelled:
            handler = self.handler
            event_loop = self.event_loop
            if self.interval:
                handler.reschedule(event_loop.timer() + self.interval)
                heappush(event_loop._scheduled, handler)
            else:
                self.event_loop._callbacks.append(self.handler)
                

class EventLoop(IObase, BaseEventLoop):
    """A pluggable event loop which conforms with the pep-3156_ API. The
event loop is the place where most asynchronous operations are carried out.

.. attribute:: io

    The I/O implementation. If not supplied, the best possible
    implementation available will be used. On posix system this is ``epoll``,
    or else ``select``. It can be any other custom implementation as long as
    it has an ``epoll`` like interface.

.. attribute:: cpubound

    If ``True`` this is a CPU bound event loop, otherwise it is an I/O
    event loop. CPU bound loops can block the loop for considerable amount
    of time.
        
.. attribute:: num_loops

    Total number of loops

.. attribute:: poll_timeout

    The timeout in seconds when polling with epol or select.

    Default: `0.5`

.. attribute:: tid

    The thread id where the eventloop is running

"""
    # Never use an infinite timeout here - it can stall epoll
    poll_timeout = 0.5

    def __init__(self, io=None, logger=None, poll_timeout=None, timer=None,
                 iothreadloop=True):
        self._impl = io or IOpoll()
        self.fd_factory = getattr(self._impl, 'fd_factory', FileDescriptor)
        self.timer = timer or default_timer
        self.poll_timeout = poll_timeout if poll_timeout else self.poll_timeout
        self.logger = logger or LOGGER
        close_on_exec(self._impl.fileno())
        self._iothreadloop = iothreadloop
        self._handlers = {}
        self._callbacks = deque()
        self._scheduled = []
        self._name = None
        self.num_loops = 0
        self._waker = self.install_waker()
    
    def __repr__(self):
        return self.name
    __str__ = __repr__
    
    @property
    def name(self):
        name = self._name if self._name else '<not running>'
        if self.cpubound:
            return 'CPU bound %s %s' % (self.__class__.__name__, name)
        else:
            return '%s %s' % (self.__class__.__name__, name)
            
    @property
    def io(self):
        return self._impl
    
    @property
    def iothreadloop(self):
        '''``True`` if this :class:`EventLoop` install itself as the event
loop of the thread where it is run.'''
        return self._iothreadloop
    
    @property
    def cpubound(self):
        return getattr(self._impl, 'cpubound', False)
    
    @property
    def running(self):
        return bool(self._name)
    
    @property
    def active(self):
        return bool(self._callbacks or self._scheduled or self._handlers)
    
    ############################################################################
    ##    PEP 3156 Methods
    def run(self):
        '''Run the event loop until nothing left to do or stop() called.'''
        if not self.running:
            self._before_run()
            try:
                while self.active:
                    try:
                        self._run_once()
                    except StopEventLoop:
                        break
            finally:
                self._after_run()
    
    def run_forever(self):
        '''Run the event loop forever.'''
        if not self.running:
            self._before_run()
            try:
                while True:
                    try:
                        self._run_once()
                    except StopEventLoop:
                        break
            finally:
                self._after_run()

    def run_once(self, timeout=None):
        """Run through all callbacks and all I/O polls once.

        Calling stop() will break out of this too.
        """
        if not self.running:
            self._before_run()
            try:
                try:
                    self._run_once(timeout)
                except StopEventLoop:
                    pass
            finally:
                self._after_run()
                
    def run_until_complete(self, future, timeout=None):
        '''Run the event loop until a :class:`Deferred` *future* is done.
Return the future's result, or raise its exception. If timeout is not
``None``, run it for at most that long;  if the future is still not done,
raise TimeoutError (but don't cancel the future).'''
        self.call_soon(future.add_both, self._raise_stop_event_loop)
        handler = None
        if timeout:
            handler = self.call_later(timeout, self._raise_stop_event_loop)
        self.run()
        if handler:
            if future.done():
                handler.cancel()
            else:
                raise TimeoutError
        result = future.result_or_self()
        if is_failure(result):
            result.raise_all()
        else:
            return result
        
    def stop(self):
        '''Stop the loop after the current event loop iteration is complete'''
        self.call_soon_threadsafe(self._raise_stop_event_loop)
    
    def call_at(self, when, callback, *args):
        '''Arrange for a ``callback`` to be called at a given time ``when``
in the future. The time is an absolute time, for relative time check
the :meth:`call_later` method.
Returns a :class:`TimedCall` with a :meth:`TimedCall.cancel` method
that can be used to cancel the call.'''
        if when > self.timer():
            timeout = TimedCall(when, callback, args)
            heappush(self._scheduled, timeout)
            return timeout
        else:
            return self.call_soon(callback, *args)
        
    def call_later(self, seconds, callback, *args):
        '''Arrange for a ``callback`` to be called at a given time in the future.
Returns a :class:`TimedCall` with a :meth:`TimedCall.cancel` method
that can be used to cancel the call. The delay can be an int or float,
expressed in ``seconds``. It is always a relative time.

Each callback will be called exactly once.  If two callbacks
are scheduled for exactly the same time, it is undefined which
will be called first.

Callbacks scheduled in the past are passed on to :meth:`call_soon` method,
so these will be called in the order in which they were
registered rather than by time due.  This is so you can't
cheat and insert yourself at the front of the ready queue by
using a negative time.

Any positional arguments after the callback will be passed to
the callback when it is called.'''
        if seconds > 0:
            timeout = TimedCall(self.timer() + seconds, callback, args)
            heappush(self._scheduled, timeout)
            return timeout
        else:
            return self.call_soon(callback, *args)
        
    def call_soon(self, callback, *args):
        '''Equivalent to ``self.call_later(0, callback, *args)``.'''
        timeout = TimedCall(None, callback, args)
        self._callbacks.append(timeout)
        return timeout
    
    def call_soon_threadsafe(self, callback, *args):
        '''Calls the given callback on the next I/O loop iteration.
It is safe to call this method from any thread at any time.
Note that this is the *only* method in :class:`EventLoop` that
makes this guarantee. all other interaction with the :class:`EventLoop`
must be done from that :class:`EventLoop`'s thread. It may be used
to transfer control from other threads to the EventLoop's thread.'''
        timeout = self.call_soon(callback, *args)
        self.wake()
        return timeout
        
    def add_reader(self, fd, callback, *args):
        """Add a reader callback.  Return a Handler instance."""
        handler = TimedCall(None, callback, args)
        fd = file_descriptor(fd)
        if fd not in self._handlers:
            self._handlers[fd] = self.fd_factory(fd, self)
        self._handlers[fd].add_reader(handler)
        return handler
    
    def add_writer(self, fd, callback, *args):
        """Add a reader callback.  Return a Handler instance."""
        handler = TimedCall(None, callback, args)
        fd = file_descriptor(fd)
        if fd not in self._handlers:
            self._handlers[fd] = self.fd_factory(fd, self)
        self._handlers[fd].add_writer(handler)
        return handler
    
    def add_connector(self, fd, callback, *args):
        handler = TimedCall(None, callback, args)
        fd = file_descriptor(fd)
        if fd not in self._handlers:
            self._handlers[fd] = self.fd_factory(fd, self)
        self._handlers[fd].add_connector(handler)
        return handler
    
    def remove_reader(self, fd):
        '''Cancels the current read callback for file descriptor fd,
if one is set. A no-op if no callback is currently set for the file
descriptor.'''
        fd = file_descriptor(fd)
        if fd in self._handlers:
            self._handlers[fd].remove_reader()
    
    def remove_writer(self, fd):
        '''Cancels the current write callback for file descriptor fd,
if one is set. A no-op if no callback is currently set for the file
descriptor.'''
        fd = file_descriptor(fd)
        if fd in self._handlers:
            self._handlers[fd].remove_writer()
            
    def remove_connector(self, fd):
        fd = file_descriptor(fd)
        if fd in self._handlers:
            self._handlers[fd].remove_connector()
    
    def add_signal_handler(self, sig, callback, *args):
        '''Whenever signal ``sig`` is received, arrange for `callback(*args)` to
be called. Returns a :class:`TimedCall` handler which can be used to cancel
the signal callback.'''
        self._check_signal(sig)
        handler = TimedCall(None, callback, args)
        prev = signal.signal(sig, handler)
        if isinstance(prev, TimedCall):
            prev.cancel()
        return handler
    
    def remove_signal_handler(self, sig):
        '''Remove the signal ``sig`` if it was installed and reinstal the
default signal handler ``signal.SIG_DFL``.'''
        self._check_signal(sig)
        handler = signal.signal(sig, signal.SIG_DFL)
        if handler:
            handler.cancel()
            return True
        else:
            return False
        
    def create_server(self, **kwargs):
        '''Create a new :class:`Server`.'''
        if self.cpubound:
            raise RuntimeError('Cannot create server from a cpubound eventloop')
        return create_server(self, **kwargs)
    
    def wake(self):
        '''Wake up the eventloop.'''
        if self.running and self._waker:
            self._waker.wake()
            
    ############################################################ NON PEP METHODS
    def call_repeatedly(self, interval, callback, *args):
        """Call a ``callback`` every ``interval`` seconds. It handles
asynchronous results. If an error occur in the ``callback``, the chain is
broken and the ``callback`` won't be called anymore."""
        return LoopingCall(self, callback, args, interval)
    
    def call_every(self, callback, *args):
        """Same as :meth:``call_repeatedly`` with the only difference that
the ``callback`` is scheduled at every loop."""
        return LoopingCall(self, callback, args)
            
    def call_now_threadsafe(self, callback, *args):
        '''Same as :meth:`call_soon_threadsafe` with the only exception that
if we are calling from the same thread of execution as this
:class:`EventLoop`, the ``callback`` is called immediately.'''
        if self.tid != current_thread().ident:
            return self.call_soon_threadsafe(callback, *args)
        else:
            self._call(callback, *args)
            
    def has_callback(self, callback):
        if callback.deadline:
            return callback in self._scheduled
        else:
            return callback in self._callbacks
        
    def install_waker(self):
        # Install event loop wake if possible
        if hasattr(self._impl, 'install_waker'):
            return self._impl.install_waker(self)
        else:
            waker = Waker()
            self.add_reader(waker, waker.consume)
            return waker

    ############################################################################
    ##    INTERNALS
    ############################################################################
    
    def _before_run(self):
        ct = setid(self)
        self._name = ct.name
        if self._iothreadloop:
            set_event_loop(self)
    
    def _after_run(self):
        self.logger.debug('Exiting %s', self)
        self._name = None
        self.tid = None
        
    def _raise_stop_event_loop(self, exc=None):
        self.logger.debug('Stopping %s', self)
        raise StopEventLoop

    def _check_signal(self, sig):
        """Internal helper to validate a signal.

        Raise ValueError if the signal number is invalid or uncatchable.
        Raise RuntimeError if there is a problem setting up the handler.
        """
        if not isinstance(sig, int):
            raise TypeError('sig must be an int, not {!r}'.format(sig))
        if signal is None:
            raise RuntimeError('Signals are not supported')
        if not (1 <= sig < signal.NSIG):
            raise ValueError('sig {} out of range(1, {})'.format(sig,
                                                                 signal.NSIG))

    def _run_once(self, timeout=None):
        timeout = timeout or self.poll_timeout
        self.num_loops += 1
        #
        # Compute the desired timeout
        if self._callbacks:
            timeout = 0
        elif self._scheduled:
            timeout = min(max(0, self._scheduled[0].deadline - self.timer()),
                          timeout)
        # poll events
        self._poll(timeout)
        #
        # append scheduled callback
        now = self.timer()
        while self._scheduled and self._scheduled[0].deadline <= now:
            self._callbacks.append(heappop(self._scheduled))
        #
        # Call the callbacks
        callbacks = self._callbacks
        todo = len(callbacks)
        call = self._call
        for i in range(todo):
            call(callbacks.popleft())
    
    def _poll(self, timeout):
        callbacks = self._callbacks
        try:
            event_pairs = self._impl.poll(timeout)
        except Exception as e:
            if self._raise_loop_error(e):
                raise
        except KeyboardInterrupt as e:
            self.logger.warning('%s stop event loop', e.__class__.__name__)
            raise StopEventLoop
        else:
            events = []
            for fd, events in event_pairs:
                if fd in self._handlers:
                    callbacks.append(partial(self._handlers[fd], events))
                else:
                    self.logger.warning('Received an event on unregistered '\
                                        'file descriptor %s' % fd)
        
    def _call(self, callback, *args):
        try:
            log_failure(callback(*args))
        except socket.error as e:
            if self._raise_loop_error(e):
                log_failure(e, msg='Exception in event loop callback.')
        except Exception as e:
            log_failure(e, msg='Exception in event loop callback.')

    def _raise_loop_error(self, e):
        # Depending on python version and EventLoop implementation,
        # different exception types may be thrown and there are
        # two ways EINTR might be signaled:
        # * e.errno == errno.EINTR
        # * e.args is like (errno.EINTR, 'Interrupted system call')
        eno = getattr(e, 'errno', None)
        if eno not in SOCKET_INTERRUPT_ERRORS:
            args = getattr(e, 'args', None)
            if isinstance(args, tuple) and len(args) == 2:
                eno = args[0]
        if eno not in SOCKET_INTERRUPT_ERRORS and self.running:
            return True


'''The :mod:`pulsar.apps.pulse` module is a django application for running
a django web site with pulsar. Add it to the list of your ``INSTALLED_APPS``::

    INSTALLED_APPS = (
        ...,
        'pulsar.apps.pulse',
        ...
    )
    
and run the site via the ``pulse`` command::

    python manage.py pulse


'''


'''Pulsar is shipped with four WSGI application handlers which facilitate the
development of server-side python web applications.

.. note::

    A **WSGI application handler** is always a callable, either a function
    or a callable object, which accepts two positional arguments:
    ``environ`` and ``start_response``. When called by the server,
    the application object must return an iterable yielding zero or more bytes. 


WsgiHandler
======================

The first and most basic application handler is the :class:`WsgiHandler`
which is a step above the :ref:`hello callable <tutorials-hello-world>`
in the tutorial. It accepts two iterables, a list of
:ref:`wsgi middleware <wsgi-middleware>` and an optional list of
:ref:`response middleware <wsgi-response-middleware>`.

.. autoclass:: WsgiHandler
   :members:
   :member-order: bysource
   
.. _wsgi-lazy-handler:

Lazy Wsgi Handler
======================

.. autoclass:: LazyWsgi
   :members:
   :member-order: bysource
   
   
.. _wsgi-middleware:

WSGI Middleware
=====================

Middleware are function or callable objects similar to :ref:`wsgi-handlers`
with the only difference that they can return ``None``. Middleware is used
in conjunction with both :class:`WsgiHandler`.
Here we introduce the :class:`Router` and :class:`MediaRouter` which handle
requests on given urls. Pulsar is shipped with
:ref:`additional wsgi middleware <wsgi-additional-middleware>` for manipulating
the environment before a client response is returned.

.. _apps-wsgi-router:

Router
~~~~~~~~~~~~~~~~~~

Next up is routing. Routing is the process of match and
parse the URL to something we can use. Pulsar provides a flexible integrated
routing system you can use for that. It works by creating a
:class:`Router` instance with its own ``rule`` and, optionally, additional
sub-routers for handling additional urls::

    class Page(Router):
        accept_content_types = ('text/html',
                                'text/plain',
                                'application/json')
        
        def get(self, request):
            "This method handle request with get-method" 
            ...
            
        def post(self, request):
            "This method handle request with post-method" 
            ...
            
    middleware = Page('/bla')
    
The ``middleware`` constructed can be used to serve ``get`` and ``post`` methods
at ``/bla``.
The :class:`Router` introduces a new element into pulsar WSGI handlers, the
:ref:`wsgi request <app-wsgi-request>`, a light-weight wrapper of the
WSGI environ.

.. autoclass:: Router
   :members:
   :member-order: bysource


Media Router
~~~~~~~~~~~~~~~~~~~~~~~~

The :class:`MediaRouter` is a spcialised :class:`Router` for serving static
files such ass ``css``, ``javascript``, images and so forth.

.. autoclass:: MediaRouter
   :members:
   :member-order: bysource
   
   
.. _WSGI: http://www.wsgi.org
'''
import os
import re
import stat
import mimetypes
from functools import partial
from email.utils import parsedate_tz, mktime_tz

from pulsar.utils.httpurl import http_date, CacheControl, remove_double_slash
from pulsar.utils.structures import AttributeDictionary, OrderedDict
from pulsar.utils.log import LocalMixin, local_property
from pulsar import Http404, PermissionDenied, HttpException, HttpRedirect, async, is_async

from .route import Route
from .utils import wsgi_request
from .content import Html
from .wrappers import WsgiResponse

__all__ = ['WsgiHandler', 'LazyWsgi', 'Router',
           'MediaRouter', 'FileRouter', 'MediaMixin']


class WsgiHandler(object):
    '''An handler for application conforming to python WSGI_.

.. attribute:: middleware

    List of callable WSGI middleware callable which accept
    ``environ`` and ``start_response`` as arguments.
    The order matter, since the response returned by the callable
    is the non ``None`` value returned by a middleware.

.. attribute:: response_middleware

    List of functions of the form::

        def ..(environ, response):
            ...

    where ``response`` is a :ref:`WsgiResponse <wsgi-response>`.
    Pulsar contains some :ref:`response middlewares <wsgi-response-middleware>`.

'''
    def __init__(self, middleware=None, response_middleware=None, **kwargs):
        if middleware:
            middleware = list(middleware)
        self.middleware = middleware or []
        self.response_middleware = response_middleware or []

    def __call__(self, environ, start_response):
        '''The WSGI callable'''
        response = None
        for middleware in self.middleware:
            response = middleware(environ, start_response)
            if response is not None:
                break
        if response is None:
            raise Http404(environ.get('PATH_INFO','/'))
        if is_async(response):
            return response.add_callback(
                        partial(self.finish, environ, start_response))
        else:
            return self.finish(environ, start_response, response)
    
    def finish(self, environ, start_response, response):
        if isinstance(response, WsgiResponse):
            for middleware in self.response_middleware:
                response = middleware(environ, response)
            start_response(response.status, response.get_headers())
        return response
    

class LazyWsgi(LocalMixin):
    '''A :ref:`wsgi handler <wsgi-handlers>` which loads the actual
handler the first time it is called. Subclasses must implement
the :meth:`setup` method.
Useful when working in multiprocessing mode when the application
handler must be a ``pickable`` instance. This handler can rebuild
its wsgi :attr:`handler` every time is pickled and un-pickled without
causing serialisation issues.'''        
    def __call__(self, environ, start_response):
        return self.handler(environ, start_response)
    
    @local_property
    def handler(self):
        '''The :ref:`wsgi application handler <wsgi-handlers>` which
is loaded via the :meth:`setup` method, once only, when first accessed.'''
        return self.setup()
    
    def setup(self):
        '''The setup function for this :class:`LazyWsgi`. Called once only
the first time this application handler is invoked. This **must** be implemented
by subclasses and **must** return a
:ref:`wsgi application handler <wsgi-handlers>`.'''
        raise NotImplementedError
    
    
def get_roule_methods(attrs):
    rule_methods = []
    for code, callable in attrs:
        if code.startswith('__') or not hasattr(callable, '__call__'):
            continue
        rule_method = getattr(callable, 'rule_method', None)
        if isinstance(rule_method, tuple):
            rule_methods.append((code, rule_method))
    return sorted(rule_methods, key=lambda x: x[1][3])
    
    
class RouterType(type):
    
    def __new__(cls, name, bases, attrs):
        rule_methods = get_roule_methods(attrs.items())
        no_rule = set(attrs) - set((x[0] for x in rule_methods))
        for base in bases[::-1]:
            if hasattr(base, 'rule_methods'):
                items = base.rule_methods.items()
            else:
                g = ((name, getattr(base, name)) for name in dir(base))
                items = get_roule_methods(g)
            base_rules = [pair for pair in items if pair[0] not in no_rule]
            rule_methods = base_rules + rule_methods
        #                
        attrs['rule_methods'] = OrderedDict(rule_methods)
        return super(RouterType, cls).__new__(cls, name, bases, attrs)
    

class Redirect(object):
    
    def __init__(self, path):
        self.path = path
        
    def response(self, environ, start_response, args):
        request = wsgi_request(environ)
        raise HttpRedirect(request.full_path(self.path))
    
    
class Router(RouterType('RouterBase', (object,), {})):
    '''A WSGI application which handle multiple
:ref:`routes <apps-wsgi-route>`. A user must implement the HTTP method
required by her application. For example if the route needs to serve a ``GET``
request, the ``get(self, request)`` method must be implemented.

:param rule: String used for creating the :attr:`route` of this :class:`Router`.
:param routes: Optionals :class:`Router` instances which are added to the
    children :attr:`routes` of this router.
:param parameters: Optional parameters for this router. They are stored in the
    :attr:`parameters` attribute.
    
.. attribute:: route

    The :ref:`Route <apps-wsgi-route>` served by this :class:`Router`.

.. attribute:: routes

    List of children :class:`Router` of this :class:`Router`.

.. attribute:: parent

    The parent :class:`Router` of this :class:`Router`.
        
.. attribute:: accept_content_types

    Tuple of content type accepted by this :class:`Router`. It can be specified
    as class attribute or during initialisation. This is an important parameter.
    It is used to set the content type of the response instance.
    Default ``None``.
    
.. attribute:: parameters

    A :class:`pulsar.utils.structures.AttributeDictionary` of parameters.
'''
    creation_count = 0
    accept_content_types = None
    _parent = None
    
    def __init__(self, rule, *routes, **parameters):
        self.__class__.creation_count += 1
        self.creation_count = self.__class__.creation_count
        if not isinstance(rule, Route):
            rule = Route(rule)
        self.route = rule
        self.routes = []
        for router in routes:
            self.add_child(router)
        self.parameters = AttributeDictionary()
        for name, rule_method in self.rule_methods.items():
            rule, method, params, count = rule_method
            rparameters = params.copy()
            handler = getattr(self, name)
            if rparameters.pop('async', False): # asynchronous method
                handler = async()(handler)
                handler.rule_method = rule_method
            router = self.add_child(Router(rule, **rparameters))
            setattr(router, method, handler)
        if 'accept_content_types' in parameters:
            self.accept_content_types = parameters.pop('accept_content_types')
        self.setup(**parameters)
        
    def setup(self, content_type=None, **parameters):
        for name, value in parameters.items():
            if not hasattr(self, name) and hasattr(value, '__call__'):
                setattr(self, name, value)
            else:
                self.parameters[name] = value
    
    @property
    def root(self):
        '''The root :class:`Router` for this :class:`Router`.'''
        if self.parent:
            return self.parent.root
        else:
            return self
     
    @property
    def parent(self):
        return self._parent
    
    def parent_property(self, name):
        value = getattr(self, name, None)
        if value is None and self._parent:
            return self._parent.parent_property(name)
        else:
            return value
        
    def content_type(self, request):
        '''Evaluate the content type for the response to the client ``request``.
The method uses the :attr:`accept_content_types` tuple of accepted content
types and the content types accepted by the client and figure out
the best match.'''
        accept = self.parent_property('accept_content_types')
        if accept:
            return request.content_types.best_match(accept)
       
    def __repr__(self):
        return self.route.__repr__()
        
    def __call__(self, environ, start_response):
        path = environ.get('PATH_INFO') or '/'
        path = path[1:]
        router_args = self.resolve(path)
        if router_args:
            router, args = router_args
            return router.response(environ, start_response, args)
        
    def resolve(self, path, urlargs=None):
        '''Resolve a path and return a ``(handler, urlargs)`` tuple or
``None`` if the path could not be resolved.'''
        urlargs = urlargs if urlargs is not None else {}
        match = self.route.match(path)
        if match is None:
            if self.route.is_leaf:
                if path.endswith('/'):
                    match = self.route.match(path[:-1])
                    if match is not None and not match:
                        return Redirect(path[:-1]), None
            else:
                if not path.endswith('/'):
                    match = self.route.match('%s/' % path)
                    if match is not None:
                        return Redirect('%s/' % path), None
            return
        if '__remaining__' in match:
            remaining_path = match['__remaining__']
            for handler in self.routes:
                view_args = handler.resolve(remaining_path, urlargs)
                if view_args is None:
                    continue
                #remaining_path = match.pop('__remaining__','')
                #urlargs.update(match)
                return view_args
        else:
            return self, match
    
    def response(self, environ, start_response, args):
        '''Once the :meth:`resolve` method has matched the correct
:class:`Router` for serving the request, this matched roter invokes this method
to actually produce the WSGI response.'''
        request = wsgi_request(environ, self, args)
        # Set the response content type
        request.response.content_type = self.content_type(request)
        method = request.method.lower()
        callable = getattr(self, method, None)
        if callable is None:
            raise HttpException(status=405,
                                msg='Method "%s" not allowed' % method)
        return callable(request)
    
    def add_child(self, router):
        '''Add a new :class:`Router` to the :attr:`routes` list. If this
:class:`Router` is a leaf route, add a slash to the url.'''
        assert isinstance(router, Router), 'Not a valid Router'
        assert router is not self, 'cannot add self to children'
        if self.route.is_leaf:
            self.route = Route('%s/' % self.route.rule)
        for r in self.routes:
            if r.route == router.route:
                r.parameters.update(router.parameters)
                return r
        if router.parent:
            router.parent.remove_child(router)
        router._parent = self
        self.routes.append(router)
        return router
        
    def remove_child(self, router):
        '''remove a :class:`Router` from the :attr:`routes` list.'''
        if router in self.routes:
             self.routes.remove(router)
             router._parent = None
        
    def link(self, *args, **urlargs):
        '''Return an anchor :class:`Html` element with the `href` attribute
set to the url of this :class:`Router`.'''
        if len(args) > 1:
            raise ValueError
        url = self.route.url(**urlargs)
        if len(args) == 1:
            text = args[0]
        else:
            text = url
        return Html('a', text, href=url)
    
    def sitemap(self, root=None):
        '''This utility method returns a sitemap starting at root.
If *root* is ``None`` it starts from this :class:`Router`.

:param request: a :ref:`wsgi request wrapper <app-wsgi-request>`
:param root: Optional url path where to start the sitemap.
    By default it starts from this :class:`Router`. Pass `"/"` to
    start from the root :class:`Router`.
:param levels: Number of nested levels to include.
:return: A list of children
'''
        if not root:
            root = self
        else:
            handler_urlargs = self.root.resolve(root[1:])
            if handler_urlargs:
                root, urlargs = handler_urlargs
            else:
                return []
        return list(self.routes)    
    
    def encoding(self, request):
        '''The encoding to use for the response. By default it
returns ``utf-8``.'''
        return 'utf-8'
    

class MediaMixin(Router):
    accept_content_types = ('application/octet-stream', 'text/css')
    cache_control = CacheControl(maxage=86400)
    
    def serve_file(self, request, fullpath):
        # Respect the If-Modified-Since header.
        statobj = os.stat(fullpath)
        content_type, encoding = mimetypes.guess_type(fullpath)
        response = request.response
        if content_type: 
            response.content_type = content_type
        response.encoding = encoding
        if not self.was_modified_since(request.environ.get(
                                            'HTTP_IF_MODIFIED_SINCE'),
                                       statobj[stat.ST_MTIME],
                                       statobj[stat.ST_SIZE]):
            response.status_code = 304
        else:
            response.content = open(fullpath, 'rb').read()
            response.headers["Last-Modified"] = http_date(statobj[stat.ST_MTIME])
        return response

    def was_modified_since(self, header=None, mtime=0, size=0):
        '''Check if an item was modified since the user last downloaded it

:param header: the value of the ``If-Modified-Since`` header. If this is None,
    simply return ``True``.
:param mtime: the modification time of the item in question.
:param size: the size of the item.
'''
        try:
            if header is None:
                raise ValueError
            matches = re.match(r"^([^;]+)(; length=([0-9]+))?$", header,
                               re.IGNORECASE)
            header_mtime = mktime_tz(parsedate_tz(matches.group(1)))
            header_len = matches.group(3)
            if header_len and int(header_len) != size:
                raise ValueError()
            if mtime > header_mtime:
                raise ValueError()
        except (AttributeError, ValueError, OverflowError):
            return True
        return False
    
    def directory_index(self, request, fullpath):
        names = [Html('a', '../', href='../', cn='folder')]
        files = []
        for f in sorted(os.listdir(fullpath)):
            if not f.startswith('.'):
                if os.path.isdir(os.path.join(fullpath, f)):
                    names.append(Html('a', f, href=f+'/', cn='folder'))
                else:
                    files.append(Html('a', f, href=f))
        names.extend(files)
        return self.static_index(request, names)
    
    def html_title(self, request):
        return 'Index of %s' % request.path
    
    def static_index(self, request, links):
        title = Html('h2', self.html_title(request))
        list = Html('ul', *[Html('li', a) for a in links])
        body = Html('div', title, list)
        doc = request.html_document(title=title, body=body)
        return doc.http_response(request)
    
    
class MediaRouter(MediaMixin):
    '''A :class:`Router` for serving static media files from a given 
directory.

:param rute: The top-level url for this router. For example ``/media``
    will serve the ``/media/<path:path>`` :class:`Route`.
:param path: Check the :attr:`path` attribute.
:param show_indexes: Check the :attr:`show_indexes` attribute.

.. attribute::    path

    The file-system path of the media files to serve.
    
.. attribute::    show_indexes

    If ``True`` (default), the router will serve media file directories as
    well as media files.
'''
    def __init__(self, rute, path, show_indexes=True):
        super(MediaRouter, self).__init__('%s/<path:path>' % rute)
        self._show_indexes = show_indexes
        self._file_path = path
        
    def get(self, request):
        paths = request.urlargs['path'].split('/')
        if len(paths) == 1 and paths[0] == '':
            paths.pop(0)
        fullpath = os.path.join(self._file_path, *paths)
        if os.path.isdir(fullpath):
            if self._show_indexes:
                return self.directory_index(request, fullpath)
            else:
                raise PermissionDenied()
        elif os.path.exists(fullpath):
            return self.serve_file(request, fullpath)
        else:
            raise Http404


class FileRouter(MediaMixin):
    
    def __init__(self, route, file_path):
        super(FileRouter, self).__init__(route)
        self._file_path = file_path
        
    def get(self, request):
        return self.serve_file(request, self._file_path)
    



import time
from datetime import date, datetime
from decimal import Decimal
import json


def totimestamp(dte):
    return time.mktime(dte.timetuple())

def totimestamp2(dte):
    return totimestamp(dte) + 0.000001*dte.microsecond

def todatetime(tstamp):
    return datetime.fromtimestamp(tstamp)


class JSONDateDecimalEncoder(json.JSONEncoder):
    """
    Provide custom serializers for JSON-RPC.
    """
    def default(self, obj):
        if isinstance(obj,datetime):
            return {'__datetime__':totimestamp2(obj)}
        elif isinstance(obj, date):
            return {'__date__':totimestamp(obj)}
        elif isinstance(obj, Decimal):
            return {'__decimal__':str(obj)}
        #elif hasattr(obj,'__dict__') and hasattr(obj,'__class__'):
        #    d = obj.__dict__.copy()
        #    d['__class__'] = obj.__class__.__name__
        #    return d
        else:
            raise ValueError("%r is not JSON serializable" % (obj,))

def date_decimal_hook(dct):
    if '__datetime__' in dct:
        return todatetime(dct['__datetime__'])
    elif '__date__' in dct:
        return todatetime(dct['__date__']).date()
    elif '__decimal__' in dct:
        return Decimal(dct['__decimal__'])
    else:
        return dct


DefaultJSONEncoder = JSONDateDecimalEncoder
DefaultJSONHook = date_decimal_hook




'''The :class:`TestSuite` is a testing framework for
asynchronous/synchronous applications and for running tests in parallel
on multiple threads or processes.
It is used for testing pulsar itself but it can be used
as a test suite for any other library.

Requirements
====================
* unittest2_ for python 2.6
* mock_ for python < 3.3

.. _apps-test-intro:

Introduction
====================

To get started with pulsar asynchronous test suite is easy. The first thing to
do is to create a python ``script`` on the top level directory of your library,
let's call it ``runtests.py``::

    from pulsar.apps import TestSuite

    if __name__ == '__main__':
        TestSuite(description='Test suite for my library',
                  modules=('regression',
                           ('examples','tests'))).start()

where ``modules`` is a tuple/list which specifies where to
:ref:`search for test cases <apps-test-loading>` to run.
In the above example the test suite will look for all python files
in the ``regression`` module (in a recursive fashion), and for modules
called ``tests`` in the ``examples`` module.

To run the test suite::

    python runtests.py
    
Type::

    python runtests.py --help
    
For a list different options/parameters which can be used when running tests.

The next step is to actually write the tests.

Writing a Test Case
===========================
Only subclasses of  ``unittest.TestCase`` are collected by this application.
When running a test, pulsar looks for two extra method: ``_pre_setup`` and
``_post_teardown``. If the former is available, it is run just before the
``setUp`` method while if the latter is available, it is run
just after the ``tearDown`` method. In addition, if the ``setUpClass``
class methods is available, it is run just before
all tests functions are run and the ``tearDownClass``, if available, is run
just after all tests functions are run.

An example test case::

    # This import is equivalent in python2.6 to
    #     import unittest2 as unittest
    # Otherwise it is the same as
    #     import unittest
    from pulsar.apps.test import unittest
    
    class MyTest(unittest.TestCase):
        
        def test_async_test(self):
            result = yield maybe_async_function()
            self.assertEqual(result, ...)
            
        def test_simple_test(self):
            self.assertEqual(1, 1)

.. note::
    
    Test functions are asynchronous, when they return a generator or a
    :class:`pulsar.Deferred`, synchronous, when they return anything else.

.. _apps-test-loading:

Loading Tests
=================

Loading test cases is accomplished via the :class:`loader.TestLoader` class. In
this context we refer to an ``object`` as a ``module`` (including a
directory module) or a ``class``.

These are the rules for loading tests:

* Directories that aren't packages are not inspected. This means that if a directory
  does not have a ``__init__.py`` file it won't be inspected.
* Any class that is a ``unittest.TestCase`` subclass is collected.
* If an object starts with ``_`` or ``.`` it won't be collected,
  nor will any objects it contains.
* If an object defines a ``__test__`` attribute that does not evaluate to
  ``True``, that object will not be collected, nor will any objects it contains.
  
The paths which will be inspected are defined by the **modules** initialisation
parameter for the :class:`TestSuite` class.
The **modules** parameter is a tuple or list of entries where each single entry
follow the following rules:

* If the entry is a string, it indicates the **dotted path** relative to the
  **root** directory (the directory of the script running the tests). For
  example::
  
      modules = ['tests', 'moretests.here']
  
* If an entry is two elements tuple, than the first element represents
  a **dotted path** relative to the **root** directory and the
  second element a **pattern** which modules (files or directories) must match
  in order to be included in the search. For example::
  
      modules = [...,
                 ('examples', 'test*')
                 ]
  
  load modules, from inside the ``examples`` module, starting with ``test``.
  
* If an entry is a three elements tuple, it is the same as the *two elements
  tuple* rule with the third element which specifies the top level **tag**
  for all tests in this entry. For example::
  
      modules = [...,
                  ('bla', '*', 'foo')
                ]

For example, the following::

    modules = ['test', ('bla', '*', 'foo'), ('examples','test*')]

loads
    
* all tests from modules in the ``test`` directory,
* all tests from the ``bla`` directory with top level tag ``foo``,
* all tests from modules which starts with *test* from the ``examples`` directory.
     
All top level modules will be added to the python ``path``.


.. _test-suite-options:

Options
==================

All standard :ref:`settings <settings>` can be applied to the test application.
In addition, the following options are testsuite-specific:

.. _apps-test-sequential:

sequential
~~~~~~~~~~~~~~~~~~~
By default, test functions within a :class:`unittest.TestCase`
are run in asynchronous fashion. This means that several test functions
may be executed at once depending on their return values.
By specifying the ``--sequential`` command line option, the :class:`TestSuite`
forces tests to be run in a sequential model, one after the other::

    python runtests.py --sequential
    
Alternatively, if you need to specify a Testcase which always runs its test functions
in a sequential way, you can use the :func:`sequential` decorator::

    from pulsar.apps.test import unittest, sequential
    
    @sequential
    class MyTestCase(unittest.TestCase):
        ...
        
        
list labels
~~~~~~~~~~~~~~~~~~~
By passing the ``-l`` or ``--list-labels`` flag to the command line, the
full list of test labels available is displayed::

    python runtests.py -l
    

test timeout
~~~~~~~~~~~~~~~~~~~
When running asynchronous tests, it can be useful to set a cap on how
long a test function can wait for results. This is what the
``--test-timeout`` command line flag does::

    python runtests.py --test-timeout 10
    
Set the test timeout to 10 seconds.


Plugins
==================
A :class:`TestPlugin` is a way to extend the test suite with additional
:ref:`options <test-suite-options>` and behaviours implemented in
the various plugin's callbacks.
There are two basic rules for plugins:

* Plugin classes should subclass :class:`TestPlugin`.
* Plugins may implement any of the methods described in the class
  :class:`result.Plugin` interface.

Pulsar ships with two plugins:

.. _bench-plugin:

Benchmark
~~~~~~~~~~~~~

.. automodule:: pulsar.apps.test.plugins.bench


.. _profile-plugin:

Profile
~~~~~~~~~~~~~

.. automodule:: pulsar.apps.test.plugins.profile


.. module:: pulsar.apps.test

API
=========

.. autoclass:: TestSuite
   :members:
   :member-order: bysource

.. automodule:: pulsar.apps.test.loader

.. automodule:: pulsar.apps.test.result

.. module:: pulsar.apps.test

Test Plugin
~~~~~~~~~~~~~~~~~~~

.. autoclass:: TestPlugin
   :members:
   :member-order: bysource
   
.. _unittest2: http://pypi.python.org/pypi/unittest2
.. _mock: http://pypi.python.org/pypi/mock
'''
__test__ = False
import logging
import time

import pulsar
from pulsar.apps import tasks
from pulsar.utils import events
from pulsar.utils.pep import ispy26, ispy33
from pulsar.utils.log import local_property

if ispy26: # pragma nocover
    try:
        import unittest2 as unittest
    except ImportError:
        print('To run tests in python 2.6 you need to install\
 the unittest2 package')
        exit(0)
else:
    import unittest
    
if ispy33: 
    from unittest import mock
else: # pragma nocover
    try:
        import mock
    except ImportError:
        print('To run tests you need to install the mock package')
        exit(0)

from .result import *
from .case import *
from .plugins.base import *
from .loader import *
from .utils import *
from .wsgi import *

def dont_run_with_thread(obj):
    actor = pulsar.get_actor()
    if actor:
        d = unittest.skipUnless(actor.cfg.concurrency=='process',
                                'Run only when concurrency is process')
        return d(obj)
    else:
        return obj


class ExitTest(Exception):
    pass


class TestVerbosity(TestOption):
    name = 'verbosity'
    flags = ['--verbosity']
    type = int
    default = 1
    desc = """Test verbosity, 0, 1, 2, 3"""
    
    
class TestTimeout(TestOption):
    name = 'test_timeout'
    flags = ['--test-timeout']
    validator = pulsar.validate_pos_int
    type = int
    default = 30
    desc = '''\
        Tests which take longer than this many seconds are timed-out
        and failed.'''


class TestLabels(TestOption):
    name = "labels"
    nargs = '*'
    validator = pulsar.validate_list
    desc = """Optional test labels to run. If not provided\
 all tests are run.

To see available labels use the -l option."""


class TestSize(TestOption):
    name = 'size'
    flags = ['--size']
    #choices = ('tiny','small','normal','big','huge')
    default = 'normal'
    desc = """Optional test size."""


class TestList(TestOption):
    name = "list_labels"
    flags = ['-l','--list-labels']
    action = 'store_true'
    default = False
    validator = pulsar.validate_bool
    desc = """List all test labels without performing tests."""
    

class TestSequential(TestOption):
    name = "sequential"
    flags = ['--sequential']
    action = 'store_true'
    default = False
    validator = pulsar.validate_bool
    desc = """Run test functions sequentially. Don't run them asynchronously."""
    

class TestSuite(tasks.TaskQueue):
    '''An asynchronous test suite which works like a task queue where each task
is a group of tests specified in a test class.

:parameter modules: An iterable over modules where to look for tests.
    Alternatively it can be a callable returning the iterable over modules.
    For example::

        suite = TestSuite(modules=('regression',
                                   ('examples','tests'),
                                   ('apps','test_*')))
        
        def get_modules(suite):
            ...

        suite = TestSuite(modules=get_modules)

    If not provided it is set as default to ``["tests"]`` which loads all
    python module from the tests module in a recursive fashion.
    Check the the :class:`TestLoader` for detailed information.

:parameter result_class: Optional class for collecting test results. By default
    it used the standard ``unittest.TextTestResult``.
:parameter plugins: Optional list of :class:`TestPlugin` instances.
'''
    name = 'test'
    cfg = pulsar.Config(apps=('tasks', 'test'),
                        loglevel='none',
                        task_paths=['pulsar.apps.test.case'],
                        plugins=(),
                        logconfig={
                                   'loggers': {
                                LOGGER.name: {'handlers': ['console_message'],
                                              'level': logging.INFO}
                                               }
                                   })
    @local_property
    def runner(self):
        '''Instance of :class:`TestRunner` driving the test case
configuration and plugins.'''
        result_class = getattr(self, 'result_class', None)
        r = unittest.TextTestRunner()
        stream = r.stream
        runner = TestRunner(self.cfg.plugins, stream, result_class)
        abort_message = runner.configure(self.cfg)
        if abort_message:
            raise ExitTest(str(abort_message))
        return runner

    def on_config(self):
        #When config is available load the tests and check what type of
        #action is required.
        modules = self.cfg.get('modules')
        # Create a runner and configure it
        runner = self.runner
        if not modules:
            modules = ['tests']
        if hasattr(modules, '__call__'):
            modules = modules(self)
        loader = TestLoader(self.root_dir, modules, runner, logger=self.logger)
        # Listing labels
        if self.cfg.list_labels:
            tags = self.cfg.labels
            if tags:
                s = '' if len(tags) == 1 else 's'
                print('\nTest labels for label{0} {1}\n'\
                      .format(s,', '.join(tags)))
            else:
                print('\nAll test labels\n')
            def _tags():
                for tag, mod in loader.testmodules(tags):
                    doc = mod.__doc__
                    if doc:
                        tag = '{0} - {1}'.format(tag,doc)
                    yield tag
            for tag in sorted(_tags()):
                print(tag)
            print('\n')
            return False
        self.local.loader = loader

    def monitor_start(self, monitor):
        '''When the monitor starts load all test classes into the queue'''
        super(TestSuite, self).monitor_start(monitor)
        loader = self.local.loader
        tags = self.cfg.labels
        try:
            self.local.tests = tests = list(loader.testclasses(tags))
            self._time_start = None
            if tests:
                self.logger.info('loaded %s test classes', len(tests))
                self.runner.on_start()
                events.fire('tests', self, tests=tests)
                monitor.cfg.set('workers', min(self.cfg.workers, len(tests)))
                self._time_start = time.time()
                for tag, testcls in self.local.tests:
                    self.backend.run('test', testcls, tag)
                monitor.event_loop.call_every(self._check_queue)
            else:
                raise ExitTest('Could not find any tests.')
        except ExitTest as e:
            print(str(e))
            monitor.arbiter.stop()
        except Exception:
            LOGGER.critical('Error occurred before starting tests',
                            exc_info=True)
            monitor.arbiter.stop()

    @classmethod
    def create_config(cls, *args, **kwargs):
        cfg = super(TestSuite, cls).create_config(*args, **kwargs)
        if cfg.params.get('plugins') is None:
            cfg.params['plugins'] = ()
        for plugin in cfg.params['plugins']:
            cfg.settings.update(plugin.config.settings)
        return cfg
    
    @pulsar.async()
    def _check_queue(self):
        runner=  self.runner
        tests = yield self.backend.get_tasks(status=tasks.READY_STATES)
        if len(tests) == len(self.local.tests):
            time_taken = time.time() - self._time_start
            for task in tests:
                runner.add(task.result)
            runner.on_end()
            runner.printSummary(time_taken)
            # Shut down the arbiter
            if runner.result.errors or runner.result.failures:
                exit_code = 1
            else:
                exit_code = 0
            raise pulsar.HaltServer(exit_code=exit_code)